{

original source from:

+---------------------------------------+-----------------+
: Klint Gore                            : "Non rhyming    :
: EMail   : kg@kgb.une.edu.au           :  slang - the    :
: Snail   : A.B.R.I.                    :  possibilities  :
: Mail      University of New England   :  are useless"   :
:           Armidale NSW 2351 Australia :     L.J.J.      :
: Fax     : +61 2 6772 5376             :                 :
+---------------------------------------+-----------------+

This unit verifies whether a user exists on an NT box

Modifications:

    Version: 1.0
       Date: 101901
     Author: Mike vanMeeteren
Description: Added new function that extracts all the users of a
             particular group, from both Local and Net sources.

}

unit netuser;

interface

uses
  Classes;

function GetGroupUserList(gName:string; var DebugInfo : string) : TStringList;
function UserExists(uname:string) : boolean;

implementation

uses
  sysutils,windows;

const
(* in windows.pas
  ERROR_ACCESS_DENIED       = 5;
  ERROR_INVALID_NAME        = 123;
  ERROR_MORE_DATA           = 234;
  ERROR_NO_SUCH_ALIAS       = 1376;

  { Invalid access to memory location. }
  ERROR_NOACCESS = 998;
  { The specified local group does not exist. }
  ERROR_NO_SUCH_ALIAS = 1376;
*)
  NERR_BASE                 = 2100;
  NERR_Success              = 0;
  NERR_GroupNotFound        = NERR_BASE + 120;
  NERR_UserNotFound         = NERR_BASE + 121;
  NERR_InvalidComputer      = NERR_BASE + 251;
  NERR_DCNotFound           = NERR_BASE + 353;

type

    TUSER_INFO_0 = record
       name             :LPWSTR;
    end;

    TGROUP_USERS_INFO_0 = record
       name             :LPWSTR;
    end;

function NetUserGetInfo(lpServer :LPWSTR;
                        UserName :LPWSTR;
                        Level    :DWORD;
                        lpBuffer :LPDWORD):DWORD;
                        stdcall;external 'netapi32.dll';

function NetGroupGetUsers(servername :LPWSTR;
                          GroupName :LPWSTR;
                          Level    :DWORD;
                          lpBuffer :LPDWORD;
                          prefmaxlen : DWORD;
                          entriesread : LPDWORD;
                          totalentries : LPDWORD;
                          resumeHandle : LPDWORD):DWORD;
                          stdcall; external 'netapi32.dll';

function NetLocalGroupGetMembers(servername :LPWSTR;
                          GroupName :LPWSTR;
                          Level    :DWORD;
                          lpBuffer :LPDWORD;
                          prefmaxlen : DWORD;
                          entriesread : LPDWORD;
                          totalentries : LPDWORD;
                          resumeHandle : LPDWORD):DWORD;
                          stdcall; external 'netapi32.dll';

function NetGroupEnum(servername :LPWSTR;
                      Level    :DWORD;
                      lpBuffer :LPDWORD;
                      prefmaxlen : DWORD;
                      entriesread : LPDWORD;
                      totalentries : LPDWORD;
                      resumeHandle : LPDWORD):DWORD;
                      stdcall; external 'netapi32.dll';

function NetLocalGroupEnum(servername :LPWSTR;
                      Level    :DWORD;
                      lpBuffer :LPDWORD;
                      prefmaxlen : DWORD;
                      entriesread : LPDWORD;
                      totalentries : LPDWORD;
                      resumeHandle : LPDWORD):DWORD;
                      stdcall; external 'netapi32.dll';


function NetGetDCName(servername:LPWSTR;
                      domainname:LPWSTR;
                      bufptr:lpdword):dword;
                      stdcall; external 'netapi32.dll';

function NetApiBufferFree(bufptr:pointer):dword;
                          stdcall; external 'netapi32.dll';

function NetApiBufferAllocate(bytecount : dword; bufptr:pointer):dword;
                             stdcall; external 'netapi32.dll';

function UserExists(uname:string) : boolean;

var
  UserName     :PChar;
  USERINFO     :^TUSER_INFO_0;

  wszUsername : array [0..255] of wchar;
  Compname : lpdword;

  GetResult : dword;

begin
  UserExists := false;

  username := stralloc(length(uname)+1);
  UserName := strpcopy(username,uname);
  multibytetowidechar(CP_ACP, 0, username, strlen(username)+1, wszUsername, sizeof(wszusername));

  GetResult := NetGetDCName(nil,nil,@compname);
  if (GetResult <> ERROR_SUCCESS) then
    CompName := nil;

  USERINFO := nil;
  GetResult := NetUserGetInfo(lpwstr(compname), @wszusername, 0, @userinfo);
  if (GetResult = ERROR_SUCCESS) then
    UserExists := true;

  strdispose(username);
  NetApiBufferFree(compname);
  NetApiBufferFree(USERINFO);
end;


{start v1.0 here}

function ErrorString(ErrorNumber : word) : string;
var
  S : string;
begin
  case ErrorNumber of
       0 : S := 'ERR_SUCCESS';
       5 : S := 'ERROR_ACCESS_DENIED';
     123 : S := 'ERROR_INVALID_NAME';
     234 : S := 'ERROR_MORE_DATA';
     998 : S := 'ERROR_NOACCESS';
    1376 : S := 'ERROR_NO_SUCH_ALIAS';
    2100 : S := 'NERR_BASE';
    2220 : S := 'NERR_GroupNotFound';
    2221 : S := 'NERR_UserNotFound';
    2351 : S := 'NERR_InvalidComputer';
    2453 : S := 'NERR_DCNotFound';
    else begin
      S := IntToStr(ErrorNumber);
    end;
  end;
  ErrorString := S;
end;

function GetGroupUserList(gName:string; var DebugInfo : string) : TStringList;
type
  TArray = array[0..1023] of pointer;
  PArray = ^TArray;
var
  MyList : TStringList;
  S : string;
  Loop : longint;
  GetResult : dword;
  Compname : lpdword;
  GROUPINFO  : PArray;
  wszGroupName    : array[0..255] of wchar;
  EntriesRead,TotalEntries,ResumeHandle : dword;
const
  CR = #13#10;
begin
  Result := nil;
  MyList := TStringList.Create;
  DebugInfo := '';
  try
    fillchar(wszGroupName,sizeof(wszGroupName),$00);
    gName := trim(gName)+#0#0;
    StringToWideChar(gName,wszGroupName,sizeof(wszGroupName));

    CompName := nil;
    GetResult := NetGetDCName(nil,nil,@compname);
    if (GetResult <> ERROR_SUCCESS) then
      CompName := nil;

    EntriesRead := 0;
    TotalEntries := 0;
    ResumeHandle := 0;

    GetResult := Error_MORE_DATA;
    while (GetResult = Error_MORE_DATA) do begin
      GroupInfo := nil;
      GetResult :=  NetLocalGroupGetMembers(lpwstr(compname), @wszGroupName, 3,
                                     @groupinfo,1024,@entriesread,
                                     @TotalEntries,@ResumeHandle);
      if (GetResult = Error_MORE_DATA) or (GetResult = Error_SUCCESS) then begin
        for Loop := 0 to EntriesRead-1 do begin
          S := WideCharToString(GroupInfo^[Loop]);
          while (pos('\',S) > 0) do
            delete(S,1,pos('\',S));
          MyList.Add(S);
        end;
      end;
      NetApiBufferFree(GROUPINFO);
    end;

    DebugInfo := 'Local: '+ErrorString(GetResult) + CR;

    EntriesRead := 0;
    TotalEntries := 0;
    ResumeHandle := 0;

    GetResult := Error_MORE_DATA;
    while (GetResult = Error_MORE_DATA) do begin
      GroupInfo := nil;
      GetResult :=  NetGroupGetUsers(lpwstr(compname), @wszGroupName, 0,
                                     @groupinfo,1024,@entriesread,
                                     @TotalEntries,@ResumeHandle);
      if (GetResult = Error_MORE_DATA) or (GetResult = Error_SUCCESS) then begin
        for Loop := 0 to EntriesRead-1 do begin
          S := WideCharToString(GroupInfo^[Loop]);
          MyList.Add(S);
        end;
      end;
      NetApiBufferFree(GROUPINFO);
    end;

    DebugInfo := DebugInfo + '  Net: '+ErrorString(GetResult);

    NetApiBufferFree(compname);
  except
    on E:Exception do begin
      DebugInfo := DebugInfo + CR+ '  Exception: '+E.Message;
    end;
  end;
  GetGroupUserList := MyList;
end;

end.

