{\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
Contents of this file are (C)Copyright 1999 by FASTechnologies Incorporated.
Any use of the contents of this file without an explicit license from FAST-
echnologies Incorporated is THEFT.  Copyright violators will be prosecuted.

        File : NTObj.pas
        Type : Unit
    Language : Delphi 2
    Revision : 1.0
      Author : Peter Sawatzki (ps)
        Date : 04/16/96
 Description : Components around NT objects

 other notes from original header:

 04/16/96 ps     first version for Delphi 2.0
 04/20/96 ps     add wait functions

 Copyright (c) 1996 Peter Sawatzki. All Rights Reserved.

    Revision : 1.01
      Author : MVM
        Date : 011499
 Description : Added $H+ so that HugeStrings does not have to be enabled
               in the projects' options.

    Revision : 1.02
      Author : MVM
        Date : 012699
 Description : Modified so that the exename can be stored seperately for
               the eventlog keys in the registry.

\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\}
{$H+}
Unit NTObj;

Interface

Uses
  SysUtils,
  Classes,
  Windows;

Type
  EKernelObject = Class(Exception);

  TKernelObject = Class(TPersistent)
  Private
    LeaseHandle: Boolean;
  Protected
    FHandle: THandle;
    Security: Pointer;
    Name: Pointer;
    Function GetHandle: THandle;
    Function CreateHandle: THandle; Virtual; Abstract;
  Public
    Destructor Destroy; Override;
    Procedure Lease (Value: THandle);
    Property Handle: THandle Read GetHandle;
  End;

  TMutex = Class(TKernelObject)
  Private
    FOpenExisting: Boolean;
  Protected
    InitialOwn: Bool;
    Function CreateHandle: THandle; Override;
  Public
    Property OpenExisting: Boolean Read FOpenExisting Write FOpenExisting;
  End;

  TEvent = Class(TKernelObject)
  Private
    ManualReset,
    InitialState: Bool;
    Function CreateHandle: THandle; Override;
  Public
    Constructor Create; Virtual;
    Procedure Reset;
    Procedure Signal;
  End;

  TSemaphore = Class(TKernelObject)
  Private
    FOpenExisting: Boolean;
  Protected
    InitialValue, MaximalValue: Integer;
    Function CreateHandle: THandle; Override;
  Public
    Property OpenExisting: Boolean Read FOpenExisting Write FOpenExisting;
  End;

  TEventLogType = (elSuccess, elInformation, elWarning, elError, elException);

  TEventLog = Class(TPersistent)
  Private
    FName: Array[0..63] Of Char;
    Procedure SetName (Const Value: String);
    Function GetName: String;
  Protected
  Public
    Constructor Create (Const AName: String);
    Function Register (Const ModuleName,Exename: String): Boolean;{1.02}
    Function UnRegister: Boolean;
    Procedure Add (AType: TEventLogType; Value: String);
    Property Name: String Read GetName Write SetName;
  End;

{- Wrappers to map callbacks to a instance.method }

{$Align Off}
  TWrapper = Record
    PopEAx: Byte;
    PushImm: Byte; InstPtr: Pointer;
    PushEAx: Byte;
    MovEAxImm: Byte; ProcPtr: Pointer;
    JmpEAx: Word;
  End;
{$Align On}

Function CreateWrapper (Instance, Proc: Pointer): TWrapper;

{- functions to wait on objects or list of objects. Currently the functions
   can wait for TThread and TEvent objects. Passing other objects is a
   nop (i.e. is like passing an event that is never signaled }

Function WaitForObjects (Const Values: Array Of TObject; WaitAll: Bool; Milliseconds: DWord): DWord;
Function WaitForOneObject(Const Values: Array Of TObject; Milliseconds: DWord): DWord;
Function WaitForAllObjects(Const Values: Array Of TObject; Milliseconds: DWord): DWord;

Function WaitForList(Value: TList; WaitAll: Bool; Milliseconds: DWord): DWord;
Function WaitForOneInList(Value: TList; Milliseconds: DWord): DWord;
Function WaitForAllInList(Value: TList; Milliseconds: DWord): DWord;

Function ReturnAddr: Pointer;

Implementation

Uses
  WinNT;

Var
  NullEvent: TEvent;

{- TKernelObject }

Destructor TKernelObject.Destroy;
Begin
  If (FHandle<>0) And Not LeaseHandle Then Begin
    CloseHandle(FHandle);
    FHandle:= 0
  End;
  Inherited Destroy
End;

Function TKernelObject.GetHandle: THandle;
Begin
  If FHandle=0 Then Begin
    FHandle:= CreateHandle;
    If FHandle=0 Then
      Raise EKernelObject.Create('Can''t create '+ClassName)
  End;
  Result:= FHandle
End;

Procedure TKernelObject.Lease (Value: THandle);
Begin
  If (FHandle<>0) And Not LeaseHandle Then
    CloseHandle(FHandle);
  FHandle:= Value;
  LeaseHandle:= FHandle<>0
End;

{- TMutex}

Function TMutex.CreateHandle: THandle;
Begin
  If OpenExisting And Assigned(Name) Then
    Result:= OpenMutex(MUTEX_ALL_ACCESS, True, Name)
  Else
    Result:= CreateMutex(Security, InitialOwn, Name)
End;

{- TEvent}

Constructor TEvent.Create;
Begin
  Inherited Create;
  ManualReset:= True;
  InitialState:= False;
End;

Function TEvent.CreateHandle: THandle;
Begin
  Result:= CreateEvent(Security, ManualReset, InitialState, Name)
End;

Procedure TEvent.Reset;
Begin
  ResetEvent(Handle)
End;

Procedure TEvent.Signal;
Begin
  SetEvent(Handle)
End;

{- TSemaphore}

Function TSemaphore.CreateHandle: THandle;
Begin
  If OpenExisting And Assigned(Name) Then
    Result:= OpenSemaphore(SEMAPHORE_ALL_ACCESS, True, Name)
  Else
    Result:= CreateSemaphore(Security, InitialValue, MaximalValue, Name)
End;

{- TEventLog }

Constructor TEventLog.Create (Const AName: String);
Begin
  Inherited Create;
  Name:= AName
End;

Function TEventLog.GetName: String;
Begin
  Result:= StrPas(FName)
End;

Procedure TEventLog.SetName (Const Value: String);
Begin
  StrPLCopy(FName, Value, SizeOf(FName)-1)
End;

{-definitions for application event logging}
Const
  {$r ntobj.res} {message file from ntobj.mc!}
  NTOBJ_EXCEPTION     = $C0000100;
  NTOBJ_INFORMATION   = $40000101;
  NTOBJ_WARNING       = $80000102;
  NTOBJ_ERROR         = $C0000103;
  NTOBJ_SUCCESS       = $00000104;

Procedure TEventLog.Add (AType: TEventLogType; Value: String);
Const
  MapEventId: Array[TEventLogType] Of Integer =
    (NTOBJ_SUCCESS, NTOBJ_INFORMATION, NTOBJ_WARNING, NTOBJ_ERROR, NTOBJ_EXCEPTION);
  MapErrorType: Array[TEventLogType] Of Integer =
    (EVENTLOG_SUCCESS, EVENTLOG_INFORMATION_TYPE, EVENTLOG_WARNING_TYPE,
     EVENTLOG_ERROR_TYPE, EVENTLOG_ERROR_TYPE);
Var
  hEventSource: THandle;
  szStrings: Array[0..0] Of PChar;
  lpStrings: PChar Absolute szStrings; {-because of buggy ReportEvent declr}
Begin
  szStrings[0]:= PChar(Value);
  hEventSource:= RegisterEventSource(Nil, FName);
  If hEventSource<>0 Then Begin
    ReportEvent(hEventSource, MapErrorType[AType], 0, MapEventId[AType], Nil,
                1, 0, lpStrings, Nil);
    DeRegisterEventSource(hEventSource)
  End
End;

Function TEventLog.Register (Const ModuleName,Exename: String): Boolean;{1.02}
Var
  SubKey: HKey;
  LogType: Integer;
  Buffer: Array[0..127] Of Char;
Begin
  Result:= False;
  LogType:= EVENTLOG_ERROR_TYPE Or EVENTLOG_WARNING_TYPE Or EVENTLOG_INFORMATION_TYPE;
  StrCopy(Buffer,'SYSTEM\CurrentControlSet\Services\EventLog\Application\');
  StrCat(Buffer, FName);
  If RegCreateKey(HKEY_LOCAL_MACHINE, Buffer, SubKey)=ERROR_SUCCESS Then Begin
    If  (RegSetValueEx(SubKey, 'EventMessageFile', 0, REG_EXPAND_SZ, @ExeName[1], Length(ModuleName))=ERROR_SUCCESS){1.02}
    And (RegSetValueEx(SubKey, 'TypesSupported', 0, REG_DWORD, @LogType, SizeOf(LogType))=ERROR_SUCCESS) Then
      Result:= True;
    RegCloseKey(SubKey)
  End
End;

Function TEventLog.UnRegister: Boolean;
Var
  Buffer: Array[0..127] Of Char;
Begin
  StrCopy(Buffer,'SYSTEM\CurrentControlSet\Services\EventLog\Application\');
  StrCat(Buffer, FName);
  Result:= RegDeleteKey(HKEY_LOCAL_MACHINE, Buffer)=ERROR_SUCCESS
End;

Function ReturnAddr: Pointer; Assembler;
Asm
  Mov EAx, [ESp+4]
End;

{-the following code creates a Wrapper that can call a certain instance}
Function CreateWrapper (Instance, Proc: Pointer): TWrapper;
Begin
  With Result Do Begin
    PopEAx:= $58;                      {Pop EAx}
    PushImm:= $68; InstPtr:= Instance; {Push Instance}
    PushEAx:= $50;                     {Push EAx}
    MovEAxImm:= $B8; ProcPtr:= Proc;   {Mov EAx, Proc}
    JmpEAx:= $E0FF;                    {Jmp EAx}
  End
End;

{- this function retrieves the handle of the object passed. }
Function GetTheObjectsHandle (Value: TObject): THandle;
Begin
  If      Value Is TKernelObject Then Result:= TKernelObject(Value).Handle
  Else If Value Is TThread Then Result:= TThread(Value).Handle
  Else Result:= NullEvent.Handle
End;

Function WaitForObjects (Const Values: Array Of TObject; WaitAll: Bool; Milliseconds: DWord): DWord;
Var
  i: Integer;
  Handles: PWOHandleArray;
Begin
  GetMem(Handles, SizeOf(THandle)*(High(Values)+1));
  Try
    For i:= 0 To High(Values) Do
      Handles^[i]:= GetTheObjectsHandle(Values[i]);
    Result:= WaitForMultipleObjects(High(Values)+1, Handles, WaitAll, Milliseconds);
  Finally
    FreeMem(Handles)
  End
End;

{- wait for one of the passed objects }
Function WaitForOneObject (Const Values: Array Of TObject; Milliseconds: DWord): DWord;
Begin
  Result:= WaitForObjects(Values, False, Milliseconds)
End;

{- wait for ALL of the passed objects }
Function WaitForAllObjects (Const Values: Array Of TObject; Milliseconds: DWord): DWord;
Begin
  Result:= WaitForObjects(Values, True, Milliseconds)
End;

Function WaitForList(Value: TList; WaitAll: Bool; Milliseconds: DWord): DWord;
Var
  i: Integer;
  Handles: PWOHandleArray;
Begin
  GetMem(Handles, SizeOf(THandle)*Value.Count);
  Try
    For i:= 0 To Value.Count-1 Do
      Handles^[i]:= GetTheObjectsHandle(TObject(Value[i]));
    Result:= WaitForMultipleObjects(Value.Count, Handles, WaitAll, Milliseconds);
  Finally
    FreeMem(Handles)
  End
End;

Function WaitForOneInList(Value: TList; Milliseconds: DWord): DWord;
Begin
  Result:= WaitForList(Value, False, Milliseconds)
End;

Function WaitForAllInList(Value: TList; Milliseconds: DWord): DWord;
Begin
  Result:= WaitForList(Value, True, Milliseconds)
End;

Initialization
  NullEvent:= TEvent.Create;
Finalization
  NullEvent.Free;
End.
