unit OURREGIS;

interface

function GetNextID : integer;
procedure GetSettings;
procedure ReadEMWACAliasLists;
procedure GetSMTPRCVSettings;
procedure SetSMTPRCVSettings;

implementation

uses
  Classes,SysUtils,Windows,Registry,SyncObjs,RCVGLBL;

const
  IMSRoot = '\SOFTWARE\EMWAC\IMS';
  SMTPRCVRoot = '\SOFTWARE\vanMeeteren\SMTPRCV';
  SMTPRSRoot = '\SYSTEM\CurrentControlSet\Services\smtprs\Parameters';
  LocalNetSubKey = '\LocalNets';
  LocalBlackListSubKey = '\LocalBlackList';
  WhiteListSubKey = '\WhiteList';
  LocalDomainSubKey = '\LocalDomains';
  RBLSubKey = '\DNSLists';
  AliasSubKey = '\Aliases';
  ListSubKey = '\Lists';

  IDKey = 'LastMsgId';
  MailSpoolKey = 'MailSpoolDir';
  LocalDomainsKey = 'DomainNamesAreLocal';
  LogOnKey = 'MailInLogEnabled';
  ProhibitRelayKey = 'ProhibitMailRelay';
  QueueDirKey = 'MailQueueDir';

  MailRootDirKey = 'MailRootDir';
  MailQueueDirKey = 'MailQueueDir';
  InLogDirKey = 'InlogDir';
  POP3LogDirKey = 'Pop3logDir';
  VerifyDomainsKey = 'VerifyDomains';
  AllowRelayKey = 'AllowRelay';
  LocalUsersKey = 'DoLocalUser';
  IMSUserListKey = 'UseIMSLists';
  NTUserDatabaseKey = 'UseNTUserDatabase';
  Plugin1UserListKey = 'UsePlugin1List';
  Plugin1ConfigFileKey = 'Plugin1ConfigFile';
  BlackListRejectionKey = 'BlackListRejectionNotice';
  NTGroupNameKey = 'NTGroupNameNotice';


  ExtendedErrorsKey = 'DoExtendedErrors';
  LocalNetsKey = 'DoLocalNets';
  MailInLogEnabledKey = 'MailInlogEnabled';
  PopAuthKey = 'DoPopAuth';
  DNSListsKey = 'DoDNSLists';
  DelayMinutesKey = 'DelayMinutes';
  DNSServerNameKey = 'Server';
  DNSHeaderTextKey = 'DenyText';
  AliasKey = 'Alias';


const
  NextIDCriticalSection : TCriticalSection = nil;

function GetNextID : integer;
var
  NextID : integer;
  Reg : TRegistry;
  DoReadFromIMS : boolean;
begin
  GetNextID := 0;
  Reg := TRegistry.Create;
  with Reg do begin
    DoReadFromIMS := true;
    RootKey := HKEY_LOCAL_MACHINE;
    try
      NextIDCriticalSection.Enter;
      if OpenKey(SMTPRCVRoot,false) then begin
        if ValueExists(IDKey) then begin
          NextID := ReadInteger(IDKey);
          DoReadFromIMS := false;
        end;
        CloseKey;
      end;
      if DoReadFromIMS then begin
        if not OpenKey(IMSRoot,false) then begin
          Reg.Free;
          exit;
        end;
        NextID := 0;
        if ValueExists(IDKey) then
          NextID := ReadInteger(IDKey);
        CloseKey;
      end;
      inc(NextID);
      if OpenKey(SMTPRCVRoot,true) then begin
        WriteInteger(IDKey,NextID);
        CloseKey;
      end else begin
        if not OpenKey(IMSRoot,false) then begin
          Reg.Free;
          exit;
        end;
        WriteInteger(IDKey,NextID);
        CloseKey;
      end;
    finally
      NextIDCriticalSection.Leave;
    end;
  end;
  Reg.Free;
  GetNextID := NextID;
end;

procedure GetSettings;
var
  NextID : integer;
  Reg : TRegistry;
  A : array[1..4096] of byte;
  BytesRead : longint;
  LoopCounter,Loop : integer;
  EMWACLocalDomains : string;
  P : integer;
  Finished : boolean;
  S : string;
  Names : TStringList;
begin
  EMWACLocalDomains := '';
  Reg := TRegistry.Create;
  with Reg do begin
    RootKey := HKEY_LOCAL_MACHINE;

    if OpenKey(IMSRoot,false) then begin

      if ValueExists(MailSpoolKey) then
        MailRootDir := ReadString(MailSpoolKey);

      if (length(MailRootDir) > 0) and (MailRootDir[length(MailRootDir)] <> '\') then
        MailRootDir := MailRootDir + '\';

      MailQueueDir := MailRootDir + MailQueueConst;
      InLogDir := MailRootDir + InLogConst;

      FixAllPaths;

      if ValueExists(LocalDomainsKey) then begin
        BytesRead := ReadBinaryData(LocalDomainsKey,A,sizeof(A));
        for Loop := 1 to BytesRead do begin
          if (A[Loop] = 0) then A[Loop] := ord(' '); {change null terminated string to spaces}
        end;
        setlength(EMWACLocalDomains,BytesRead);
        move(A,EMWACLocalDomains[1],BytesRead);
      end;

      CloseKey;
    end;

    Finished := false;
    while not Finished do begin
      P := pos(' ',EMWACLocalDomains);
      if (P > 0) then begin
        S := copy(EMWACLocalDomains,1,P-1);
        delete(EMWACLocalDomains,1,P);
      end else begin
        S := EMWACLocalDomains;
        finished := true;
      end;
      Trim(S);
      if (S <> '') then begin
        LocalDomainList.Add(S);
      end;
    end;

    if OpenKey(SMTPRSRoot,false) then begin

      if ValueExists(LogOnKey) then
        MailInLogEnabled  := ReadBool(LogOnKey);

      if ValueExists(QueueDirKey) then
        MailQueueDir := MailRootDir + ReadString(QueueDirKey);

      CloseKey;
    end;
  end;
  Reg.Free;
end;

procedure ReadEMWACAliasLists;
var
  Reg : TRegistry;
  LoopCounter,Loop : integer;
  P : integer;
  Finished : boolean;
  S : string;
  Names : TStringList;
begin
  Reg := TRegistry.Create;
  with Reg do begin
    RootKey := HKEY_LOCAL_MACHINE;
    Names := TStringList.Create;
    LocalUserList.Clear;
    if OpenKey(IMSRoot+AliasSubKey,false) then begin {0.33}
      Names.Clear;
      GetKeyNames(Names);
      CloseKey;
      for LoopCounter := 0 to (Names.Count - 1) do begin
        if OpenKey(IMSRoot+AliasSubKey+'\'+Names[LoopCounter],false) then begin
          if ValueExists(AliasKey) then begin
            S := Trim(ReadString(AliasKey));
            if (S <> '') then begin
              LocalUserList.Add(S);
            end;
          end;
          CloseKey;
        end;
      end;
    end;

    if OpenKey(IMSRoot+ListSubKey,false) then begin {0.35}
      Names.Clear;
      GetKeyNames(Names);
      CloseKey;
      for LoopCounter := 0 to (Names.Count - 1) do begin
        LocalUserList.Add(Names[LoopCounter]);
        LocalUserList.Add(Names[LoopCounter]+'-request');{0.37}
      end;
    end;

    Names.Free;

  end;
  Reg.Free;
end;

procedure GetSMTPRCVSettings;
var
  Reg : TRegistry;
  Names : TStringList;
  LoopCounter : integer;
  S,S2 : string;
begin
  Reg := TRegistry.Create;
  with Reg do begin
    RootKey := HKEY_LOCAL_MACHINE;

    if OpenKey(SMTPRCVRoot,false) then begin

      if ValueExists(MailRootDirKey) then
        MailRootDir := ReadString(MailRootDirKey);

      if ValueExists(MailQueueDirKey) then
        MailQueueDir := ReadString(MailQueueDirKey);

      if ValueExists(InLogDirKey) then
        InLogDir := ReadString(InLogDirKey);

      if ValueExists(POP3LogDirKey) then
        POP3LogDir := ReadString(POP3LogDirKey);

      if ValueExists(VerifyDomainsKey) then
        VerifyDomains := ReadBool(VerifyDomainsKey);

      if ValueExists(AllowRelayKey) then
        AllowRelay := ReadBool(AllowRelayKey);

      if ValueExists(LocalNetsKey) then
        LocalNets := ReadBool(LocalNetsKey);

      if ValueExists(LocalUsersKey) then
        LocalUsers := ReadBool(LocalUsersKey);

      if ValueExists(IMSUserListKey) then
        IMSUserList := ReadBool(IMSUserListKey);

      if ValueExists(NTUserDatabaseKey) then
        NTUserDatabase := ReadBool(NTUserDatabaseKey);

      if ValueExists(Plugin1UserListKey) then
        Plugin1UserList := ReadBool(Plugin1UserListKey);

      if ValueExists(Plugin1ConfigFileKey) then
        Plugin1ConfigFile := ReadString(Plugin1ConfigFileKey);

      if ValueExists(BlackListRejectionKey) then
        BlackListRejection := ReadString(BlackListRejectionKey);

      if ValueExists(NTGroupNameKey) then
        NTGroupName := ReadString(NTGroupNameKey);

      if ValueExists(ExtendedErrorsKey) then
        ExtendedErrors := ReadBool(ExtendedErrorsKey);

      if ValueExists(MailInLogEnabledKey) then
        MailInLogEnabled := ReadBool(MailInLogEnabledKey);

      if ValueExists(PopAuthKey) then
        PopAuth := ReadBool(PopAuthKey);

      if ValueExists(DNSListsKey) then
        DNSLists := ReadBool(DNSListsKey);

      if ValueExists(DelayMinutesKey) then
        DelayMinutes := ReadInteger(DelayMinutesKey);

      CloseKey;
    end;

    Names := TStringList.Create;

    if OpenKey(SMTPRCVRoot+LocalNetSubKey,false) then begin
      LocalNetList.Clear;
      Names.Clear;
      GetValueNames(Names);
      for LoopCounter := 0 to (Names.Count - 1) do begin
        if ValueExists(Names[LoopCounter]) then begin
          S := Trim(ReadString(Names[LoopCounter]));
          if (S <> '') then
            LocalNetList.Add(S);
        end;
      end;
      CloseKey;
    end;

    if OpenKey(SMTPRCVRoot+LocalBlackListSubKey,false) then begin
      LocalBlackList.Clear;
      Names.Clear;
      GetValueNames(Names);
      for LoopCounter := 0 to (Names.Count - 1) do begin
        if ValueExists(Names[LoopCounter]) then begin
          S := Trim(ReadString(Names[LoopCounter]));
          if (S <> '') then
            LocalBlackList.Add(S);
        end;
      end;
      CloseKey;
    end;

    if OpenKey(SMTPRCVRoot+WhiteListSubKey,false) then begin
      WhiteList.Clear;
      Names.Clear;
      GetValueNames(Names);
      for LoopCounter := 0 to (Names.Count - 1) do begin
        if ValueExists(Names[LoopCounter]) then begin
          S := Trim(ReadString(Names[LoopCounter]));
          if (S <> '') then
            WhiteList.Add(S);
        end;
      end;
      CloseKey;
    end;

    if OpenKey(SMTPRCVRoot+LocalDomainSubKey,false) then begin
      LocalDomainList.Clear;
      Names.Clear;
      GetValueNames(Names);
      for LoopCounter := 0 to (Names.Count - 1) do begin
        if ValueExists(Names[LoopCounter]) then begin
          S := Trim(ReadString(Names[LoopCounter]));
          if (S <> '') then
            LocalDomainList.Add(S);
        end;
      end;
      CloseKey;
    end;

    if OpenKey(SMTPRCVRoot+RBLSubKey,false) then begin
      DNSServers.Clear;
      DNSHeaderText.Clear;
      Names.Clear;
      GetKeyNames(Names);
      CloseKey;
      for LoopCounter := 0 to (Names.Count - 1) do begin
        if OpenKey(SMTPRCVRoot+RBLSubKey+'\'+Names[LoopCounter],false) then begin
          if ValueExists(DNSServerNameKey) then begin
            S := Trim(ReadString(DNSServerNameKey));
            S2 := '';
            if ValueExists(DNSHeaderTextKey) then
              S2 := Trim(ReadString(DNSHeaderTextKey));
            if (S2 = '') then S2 := S;
            if (S <> '') then begin
              DNSServers.Add(S);
              DNSHeaderText.Add(S2);
            end;
          end;
          CloseKey;
        end;
      end;
    end;
    Names.Free;
  end;
  Reg.Free;
end;

procedure SetSMTPRCVSettings;
var
  Reg : TRegistry;
  Names : TStringList;
  LoopCounter : integer;
  S,S2 : string;
begin
  Reg := TRegistry.Create;
  with Reg do begin
    RootKey := HKEY_LOCAL_MACHINE;

    if OpenKey(SMTPRCVRoot,true) then begin

      WriteString(MailRootDirKey,MailRootDir);
      WriteString(MailQueueDirKey,MailQueueDir);
      WriteString(InLogDirKey,InLogDir);
      WriteString(POP3LogDirKey,POP3LogDir);
      WriteBool(VerifyDomainsKey,VerifyDomains);
      WriteBool(AllowRelayKey,AllowRelay);
      WriteBool(LocalNetsKey,LocalNets);
      WriteBool(LocalUsersKey,LocalUsers);
      WriteBool(IMSUserListKey,IMSUserList);
      WriteBool(NTUserDatabaseKey,NTUserDatabase);
      WriteBool(Plugin1UserListKey,Plugin1UserList);
      WriteString(Plugin1ConfigFileKey,Plugin1ConfigFile);
      WriteString(BlackListRejectionKey,BlackListRejection);
      WriteString(NTGroupNameKey,NTGroupName);
      WriteBool(ExtendedErrorsKey,ExtendedErrors);
      WriteBool(MailInLogEnabledKey,MailInLogEnabled);
      WriteBool(PopAuthKey,PopAuth);
      WriteBool(DNSListsKey,DNSLists);
      WriteInteger(DelayMinutesKey,DelayMinutes);

      CloseKey;
    end;

    Names := TStringList.Create;

    if OpenKey(SMTPRCVRoot+LocalNetSubKey,true) then begin
      Names.Clear;
      GetValueNames(Names);
      for LoopCounter := 0 to (Names.Count - 1) do begin
        DeleteValue(Names[LoopCounter]);
      end;
      for LoopCounter := 0 to LocalNetList.Count-1 do begin
        S := IntToStr(LoopCounter);
        while (length(S) < 3) do S := '0' + S;
        WriteString('NET'+S,LocalNetList[LoopCOunter]);
      end;
      CloseKey;
    end;

    if OpenKey(SMTPRCVRoot+LocalBlackListSubKey,true) then begin
      Names.Clear;
      GetValueNames(Names);
      for LoopCounter := 0 to (Names.Count - 1) do begin
        DeleteValue(Names[LoopCounter]);
      end;
      for LoopCounter := 0 to LocalBlackList.Count-1 do begin
        S := IntToStr(LoopCounter);
        while (length(S) < 3) do S := '0' + S;
        WriteString('NET'+S,LocalBlackList[LoopCOunter]);
      end;
      CloseKey;
    end;

    if OpenKey(SMTPRCVRoot+WhiteListSubKey,true) then begin
      Names.Clear;
      GetValueNames(Names);
      for LoopCounter := 0 to (Names.Count - 1) do begin
        DeleteValue(Names[LoopCounter]);
      end;
      for LoopCounter := 0 to WhiteList.Count-1 do begin
        S := IntToStr(LoopCounter);
        while (length(S) < 3) do S := '0' + S;
        WriteString('NET'+S,WhiteList[LoopCOunter]);
      end;
      CloseKey;
    end;

    if OpenKey(SMTPRCVRoot+LocalDomainSubKey,true) then begin
      Names.Clear;
      GetValueNames(Names);
      for LoopCounter := 0 to (Names.Count - 1) do begin
        DeleteValue(Names[LoopCounter]);
      end;
      for LoopCounter := 0 to LocalDomainList.Count-1 do begin
        S := IntToStr(LoopCounter);
        while (length(S) < 3) do S := '0' + S;
        WriteString('Domain'+S,LocalDomainList[LoopCOunter]);
      end;
      CloseKey;
    end;

    if OpenKey(SMTPRCVRoot+RBLSubKey,true) then begin
      Names.Clear;
      GetKeyNames(Names);
      for LoopCounter := 0 to (Names.Count - 1) do begin
        DeleteKey(Names[LoopCounter]);
      end;
      for LoopCounter := 0 to DNSServers.Count-1 do begin
        S := IntToStr(LoopCounter);
        while (length(S) < 3) do S := '0' + S;
        S := '\DNSList'+S;
        if OpenKey(SMTPRCVRoot+RBLSubKey+S,true) then begin
          WriteString(DNSServerNameKey,DNSServers[LoopCounter]);
          if (LoopCounter < DNSHeaderText.Count) then
            WriteString(DNSHeaderTextKey,DNSHeaderText[LoopCounter])
          else
            WriteString(DNSHeaderTextKey,DNSServers[LoopCounter]);
        end;
      end;
      CloseKey;
    end;
    Names.Free;
  end;
  Reg.Free;
end;

initialization
  NextIDCriticalSection := TCriticalSection.Create;
finalization
  NextIDCriticalSection.Free;
end.

