unit svcmain;

interface

Uses
  SERVROOT;

type
  TSMTPService = class(TService)
  private
    Finished : boolean;
  Protected
    procedure Execute; Override;
    procedure Advertise; Override;
    procedure Start; Override;
    procedure Stop; Override;
  end;

implementation

uses
  SysUtils,Classes,Windows,NTObj,PLUGIN1,NETUSER,
  ServerWinshoeSMTP,{ANTIRLY,removed 0.36}SyncObjs,
  OURREGIS,RCVGLBL,SUBNET;

type
  TServer = object
    private
      MyOwner : TSMTPService;
      Server: TWinshoeSMTPServer;
      LogFileCritSection : TCriticalSection;
      PopAuthCritSection : TCriticalSection;
      function  Initialize(Owner : TSMTPService) : boolean;
      procedure Finalize;
      function  StartServer : boolean;
      procedure StopServer;
      procedure ServerNextIDEvent(var ID: Integer;
                                  var Handled: Boolean);
{
      function  ServerNTUserEvent(const Name : string) : boolean;
}
      procedure ServerPopAuth(const IPAddy: String; var Valid,
                              Handled: Boolean);
      procedure ServerLogFile(const logFileEntry: String);
  end;

var
  Server : TServer;

function PathExistsForReadWrite(Path : string) : boolean;
{
Renamed from PathExists in 4.46.
}
var
  Test : file;
begin
  if (Path = '') then
    PathExistsForReadWrite := false
  else begin
{$I-}
    Path := FixDirBackSlash(Path);
    assign(Test,Path+'smtprcvt.tmp');
    rewrite(Test);
    close(Test);
    erase(Test);
    PathExistsForReadWrite := (IOresult = 0);
{$I+}
  end;
end; { PathExistsForReadWrite }

function CreatePath(Path : string) : boolean;{4.29}
var
  OK : boolean;
  InPath : string;
  OldPath : string;
  TempPath : string;
  MakePath : string;
  Done : boolean;
begin
  CreatePath := false;
  if (Path = '') then exit; { otherwise causes an exception }
  if PathExistsForReadWrite(Path) then begin
    CreatePath := true;
    exit;
  end;
  OK := true;
  InPath := Path;
{$I-}
  Path := FixDirBackSlash(path);
  TempPath := '';
  try
    GetDir(0,OldPath);
    if (pos(':',Path) = 2) then begin
      TempPath := copy(Path,1,3);
      delete(Path,1,3);
      chdir(TempPath);
      OK := (ioresult = 0);
    end else if (pos('\\',Path) = 1) then begin
      OK := false;
      TempPath := '\\';
      delete(path,1,2);
      if (pos('\',Path) > 0) then begin
        TempPath := TempPath + copy(Path,1,pos('\',Path));
        delete(Path,1,pos('\',Path));
      end;
      if (pos('\',Path) > 0) then begin
        TempPath := TempPath + copy(Path,1,pos('\',Path));
        delete(Path,1,pos('\',Path));
        OK := true;
      end;
      if OK then begin
        chdir(TempPath);
        OK := (ioresult = 0);
      end;
    end;
    Done := not OK;
    MakePath := '';
    while ((not Done) and OK) do begin
      MakePath := MakePath + copy(Path,1,pos('\',Path));
      if (MakePath <> '') then begin
        if (not PathExistsForReadWrite(MakePath)) then begin
          mkdir(MakePath);
          OK := (ioresult = 0);
        end;
        delete(Path,1,pos('\',Path));
        Done := (pos('\',Path) = 0);
      end else
        Done := true;
    end;
  except
  end;
  chdir(OldPath);
  CreatePath := PathExistsForReadWrite(InPath);
  InPath := '';
  OldPath := '';
  TempPath := '';
  MakePath := '';
end;

function  TServer.Initialize(Owner : TSMTPService) : boolean;
begin
  LogFileCritSection := TCriticalSection.Create;
  PopAuthCritSection := TCriticalSection.Create;

  LocalNetList := TStringList.Create;
  LocalBlackList := TStringList.Create;
  WhiteList := TStringList.Create;
  LocalUserList := TStringList.Create;
  SkipUserVerificationList := TStringList.Create;
  LocalDomainList := TStringList.Create;
  DNSServers := TStringList.Create;
  DNSHeaderText := TStringList.Create;

  MyOwner := Owner;
  if StartServer then begin
    Initialize := true;
  end else begin
    Initialize := false;
  end;
end;

procedure TServer.Finalize;
begin
  StopServer;
  LocalNetList.Free;
  LocalBlackList.Free;
  WhiteList.Free;
  LocalUserList.Free;
  SkipUserVerificationList.Free;
  LocalDomainList.Free;
  DNSServers.Free;
  DNSHeaderText.Free;
  PopAuthCritSection.Free;
  LogFileCritSection.Free;
end;

function TServer.StartServer : boolean;
var
  FileName : string;
  S,S2 : string;
  P : integer;
  Loop : integer;
  NetMask : PSubNet;
  NTUserList : TStringList;
  Junk : string;
begin
  StartServer := false;
  Server := TWinshoeSMTPServer.Create(nil);
  with Server do begin {set defaults}
    Active := false;
    BoundIP := '';
    DNSLists.Clear;
    DoDNSLists := false;
    DoPOPAuth := false;
    DoVerifyDomain := false;
    ForwardMail := false;
    CheckUser := false;
    HeaderText.Clear;
    LocalDomains.Clear;
    LocalNets.Clear;
    LocalBlackList.Clear;
    LocalBlackListNotice := 'local deny list';
    WhiteList.Clear;
    LocalUserList.Clear;
    SkipUserVerificationList.Clear;
    LogDebug := false;
    MailWriteDir := '';
    Port := 25;
    SessionTimeOut := 0;
    OnNextIDEvent := ServerNextIDEvent;
{   OnNTUserEvent := ServerNTUserEvent; 0.42}
    OnPopAuth := ServerPopAuth;
    OnLogFileEvent := ServerLogFile;
  end;

  {get settings from SMTPRS and EMWAC IMS registry settings}
  GetSettings;

  {antirelay.ini stuff}
(* removed in 0.36
  FileName := ExtractFilePath(paramstr(0))+'antirly.ini';
  ReadAntiRelayIni(FileName);
*)

  {try to get it from our settings}
  GetSMTPRCVSettings;

  FixAllPaths;

  if IMSUserList then
    ReadEMWACAliasLists;

  if Plugin1UserList then {0.36}
    ReadUsersFromPlugin1(Plugin1ConfigFile);

  Server.MailWriteDir := FixDirBackSlash(MailQueueDir);

  for Loop := 0 to LocalDomainList.Count - 1 do
    Server.LocalDomains.Add(LocalDomainList[Loop]);

  if LocalNets then begin
    for Loop := 0 to LocalNetList.Count-1 do begin
      NetMask := ParseSubNet(LocalNetList[Loop]);
      if (NetMask <> nil) then
        Server.LocalNets.Add(NetMask);
    end;
  end;

  for Loop := 0 to LocalBlackList.Count-1 do begin {0.41}
    NetMask := ParseSubNet(LocalBlackList[Loop]);
    if (NetMask <> nil) then
      Server.LocalBlackList.Add(NetMask);
  end;

  for Loop := 0 to WhiteList.Count-1 do begin {0.41}
    NetMask := ParseSubNet(WhiteList[Loop]);
    if (NetMask <> nil) then
      Server.WhiteList.Add(NetMask);
  end;

  if NTUserDatabase then begin
    NTUserList := GetGroupUserList(NTGroupName,Junk);
    for Loop := 0 to NTUserList.Count-1 do begin
      LocalUserList.Add(NTUserList[Loop]);
    end;
    NTUserList.Free;
  end;

  for Loop := 0 to LocalUserList.Count-1 do
    Server.LocalUsers.Add(LocalUserList[Loop]);

  for Loop := 0 to SkipUserVerificationList.Count-1 do
    Server.SkipUserVerify.Add(SkipUserVerificationList[Loop]);
  for Loop := 0 to DNSServers.Count-1 do
    Server.DNSLists.Add(DNSServers[Loop]);
  for Loop := 0 to DNSHeaderText.Count-1 do
    Server.HeaderText.Add(DNSHeaderText[Loop]);
  Server.DoDNSLists := DNSLists;
  Server.DoPopAuth := PopAuth;
  Server.DoVerifyDomain := VerifyDomains;
  Server.CheckUser := LocalUsers;
  Server.ForwardMail := AllowRelay;
  Server.ExtendedErrorRecording := ExtendedErrors;
  Server.LocalBlackListNotice := BlackListRejection;

  {check for inbound directory, create it if it doesn't exist}
  if PathExistsForReadWrite(MailQueueDir) then begin
    Server.Active := true;
    StartServer := true;
  end else begin
    CreatePath(MailQueueDir);
    Server.Active := PathExistsForReadWrite(MailQueueDir);
  end;

  StartServer := Server.Active;
end;

procedure TServer.StopServer;
begin
  Server.Free;
end;

procedure TServer.ServerNextIDEvent(var ID: Integer;
                                    var Handled: Boolean);
begin
  ID := GetNextID;
  Handled := true;
end;

(* 0.42
function  TServer.ServerNTUserEvent(const Name : string) : boolean;
begin
  ServerNTUserEvent := false;
  if NTUserDatabase then {0.36}
    ServerNTUserEvent := VerifyNTUser(Name);
end;
*)

procedure TServer.ServerLogFile(const logFileEntry: String);
var
  FileName : string;
  F : system.text;
begin
  if not MailInLogEnabled then exit;
  try
    if not PathExistsForReadWrite(InLogDir) then exit;
    FileName := FormatDateTime('yymmdd',Now);
    FileName := '1'+ FileName+'.LOG'; {Y2K bug}
    FileName := FixDirBackSlash(InLogDir)+FileName;
    try
{$I-}
      try
        LogFileCritSection.Enter;
        if FileExists(FileName) then begin
          system.assign(F,FileName);
          system.append(F);
        end else begin
          system.assign(F,FileName);
          system.rewrite(F);
        end;
        writeln(F,LogFileEntry);
      finally
        system.close(F);
        LogFileCritSection.Leave;
      end;
{$I+}
    except
    end;
  except
  end;
end;

procedure TServer.ServerPopAuth(const IPAddy: String; var Valid,
                                Handled: Boolean);
var
  LoadedFile : TStringList;
  S : string;
  Loop : longint;
  P,Q : integer;
  TimeThen : TDateTime;
  FileName : string;
  Stream: TStream;
begin
  Handled := true;
  Valid := false;
  FileName := FormatDateTime('yymmdd',Now);
  FileName := '1'+ FileName+'.LOG'; {Y2K bug}
  FileName := FixDirBackSlash(Pop3LogDir)+FileName;
  if FileExists(FileName) then begin
    LoadedFile := TStringList.Create;
    try
      try
        PopAuthCritSection.Enter;
{$I-}
        Stream := TFileStream.Create(FileName, fmOpenRead or fmShareDenyNone);
        LoadedFile.LoadFromStream(Stream);
        Stream.Free;
{$I+}
      finally
        PopAuthCritSection.Leave;
      end;

      for Loop := 0 to LoadedFile.Count-1 do begin
        S := LoadedFile[Loop];
        if (pos('PASS **** OK',S) > 0) and
           (pos('-ERR',S) = 0) and {from antirelay, dunno what this does}
           (pos(IPAddy,S) > 0) then begin
          P := pos('[',S);
          Q := pos(']',S);
          if (P > 0) and (Q > 0) then begin
            S := copy(S,P+1,Q-P-1);
            P := pos(':',S);
            if (P > 0) then delete(S,1,P);
            TimeSeparator := ':';
            TimeThen := StrToTime(S); {sysutils}
            if (Time-TimeThen) <= (1/24/60)*DelayMinutes then begin
              Valid := true;
              LoadedFile.Free;
              exit;
            end;
          end;
        end;
      end;
    except
    end;
    LoadedFile.Free;
  end;
end;

{===================================================================}
{===================================================================}
{===================================================================}
{===================================================================}
{===================================================================}
{===================================================================}

procedure TSMTPService.Start;
begin
  if not Server.Initialize(self) then Terminate;
  Finished := false;
end;

const
  LoopDelay = 500;

procedure TSMTPService.Execute;
var
  WaitResult : dword;
begin
  while not Finished do begin
    WaitResult := WaitForOneObject([StopEvent], LoopDelay);
    if (WaitResult = WAIT_OBJECT_0) then begin
      Finished := true;
    end;
  end;
end;

procedure TSMTPService.Stop;
begin
  Server.Finalize;
end;

procedure TSMTPService.Advertise;
begin
  writeln('MMS (Mike''s Mail System) '+Productname+' Rev '+VersionString);
end;

end.



