////////////////////////////////////////////////////////////////////////////////
//   Server WinShoeSMTP
//
//
////////////////////////////////////////////////////////////////////////////////
//  Based on RFC 821 - The SMTP Protocol
// ..
// Author Ray Malone
// Version .10 Started July 10, 1999 Completd July 30, 1999
//
// the WinshoeSMPT server is based on TWinshoeSMTPListener  which is an abstract
// component that just listens on the port and parses the received commands. It
// calls abstract procedures for each command; Events for each of these
// comands are defined in the listener.
// The TWinshoeSPTPServer decends from the TWinshoeSMTPListener and implements
// The commands and events.
// ON receipt of a commmand the DoCommandXXXX procedure is called and if assigned
// it calls the related event. if the event is not assigned then the default
// behavior is run.
// Each event has the boolean variable named Handled. if Handled is set to True
// in the event handler default processing is not done.
//
//------------------------------------------------------------------------------
// The simplest Mail transaction is as follows
// An SMTP Client makes a connection.
//
// The Doconnect sends the following reply
//      220 '+ComputerName +' Simple Mail Transfer Service Ready
// The Client then sends a HELO command Which consists of the HELO command
//  and the Clients Domain Name
// The DoCommandHelo then replies
//        '250 '+ SMTPDomainName
//  Where ComputerName is the name returned by Win function GetComputerName
//
//  The Client then sends the MAIL command which contains the From address
//    For example: MAIL From: "Charlie Smith" <char@hisdomain.com>
//        or     MAIL From: char@hisdomain.com
//   The Mail comannd argument is stored in  fFromStr which is th FromStr property
//   if the address part is valid ( that is chars, @ chars dot chars) the server
//   Replies 250 OK
// The Client then sends one or more RCPT commans in any of the following forms:
//        RCPT To: "Bill Jones" <bill@hisdomain.com> (this is a comment)
//        RCPT To: "Bill Jones" <bill@hisdomain.com>
//        RCPT To: <bill@hisdomain.com>
//        RCPT To: bill@hisdomain.com
//  if the addess part is not valid( that is chars, @ chars dot chars) the server
//  replies 501 Syntax error
//  if the address part is valid it checks to see if the domain is local. if not
//  the reply is 250 (should be 251) that it will forward teh mail.
//  if the domain is local a check is made to see if the mail box (User Id) is
//  valid. if it is not then a 550 ERR invalid mail box error is returned.

//   The RCPT comannd arguments is stored in  fRecipients tRecipientList
//  this list contains the Addrsss part of the RCPT command as the Addresss
//  Field. A Local Mail boolean, and a Local client boolean. if both the
//  LocalMail and LocalClient are true, the path to the recipients mailbox is
//  stored in the MailBoxDir field.
//  Mail to Be forwared only has the address field set.

// The Client next sends the data command.
//   The server responds 354 Start Mail Input; end with <CRLF>.<CRLF>
// The client then sends text lines of the Email document which are stored in
//   the FData.StringList
//  When the <CRLF>.<CRLF> is received by the SErver it then stores the
//  DoCommandData then Calls WriteMailToUserDirOrForward whick drops local mail
//  to the Users MailBox or writes it to the  Pickup dir to be forwarded.
//
//  when it is finished.
//  DoCommandData then Sends the folowing reply
//   250 OK
//  At this point the client can call quit or start another Mail command
//-----------------------------------------------------------------------------
// Revision History
// 10/15/99
// 0.01 Modification to handle  extra Period and blank line
// Supplied by Roman Evstyunichev... thanks.
//
// 10/15/99
// 0.01 Revised Writetodisk to make sure To: is the intended Recipient for To
// CC and BC do not depend on x-receiver
////////////////////////////////////////////////////////////////////////////////
{
v0.11  Added logging to the IMS INLOG directory, same format as they use.
       (554 RBL rejections should show up here too)
v0.12  Fixed bug in the localnet masking routine that allowed most IP
       addresses to appear local
v0.13-0.15
       Fixed a difference between the way SMTPRS and Winshoes deals with
       the terminating period that stops the DATA transfer.  (This caused
       no non-local mail to get delivered).
v0.16  Fixed address parsing engine to correctly deal with !, % and :
       This then correctly parses and address to be a relay hack, and
       allows the server to reject the address.
v0.17  Corrected startup bug that caused service to crash when started on
       some NT4 workstations. (in mainline)
v0.18  Made MAIL FROM: <> reply RFC 2505 compatible (server now no longer
       cares what's there and responds with a 250 OK regardless).
v0.19  When IP fails to rDNS, the header says unverified instead of a blank
       spot.
v0.20  When local IP fails to rDNS, the server name changes to an IP
       instead of blank, fixing message-id problems
v0.21  Deleted an unneccesary check for localdomain
v0.22  Trying to gain some speed on the write for large files
v0.23  OK, that didn't work :)
v0.24  Removed date insertion when another received line is found
       to track down a possible bug where Netscape and Outlook puke when
       downloading mail.
v0.25  Fixed bonehead Date bug.
v0.26  3 different ways to kill an RBL listed server connection now:
       1) immediate termination via a 421 command
       2) any rcpt to: and data command result in a 554
       3) first rcpt to: results in a 554 and a killed connection
       Also fixed so that no valid RCPT results in a 503 No valid recipients specified
v0.27  Control values now read from our own registry settings.  A
       control panel app is included.
v0.28  By some miracle I discovered that if two threads hit the server at the
       exact same time, the RBL info can pass from one thread to the other.
v0.29  Fix in 0.28 removed access denied log file entries, added back in.
v0.30  Fix in 0.29 caused abstract method crash.
v0.31  After a DATA command finishes, flush the message.  Apparently some
       mailing list servers don't issue an RSET before starting the next message.
v0.32  Trying to make servername resolve correctly.
v0.33  Added localuser deny (notify MTU when user doesn't exist).  Pulled from
       the alias listing for IMS. (probably won't work for list aliases)
v0.34  Fixed time offset notation by adding calc for daylight savings
v0.35  Added TCriticalSections around log file reads/writes to make multithread
       safe.  Also added optional error logging to the inlog which adds errors
       for invalid local names and relay attempts.  Made localuser deny (0.33)
       list compatible.
v0.36  Added capability to use plugin1's config file to extract user list and
       ability to check NT user database for user name (Thanks Klint Gore).
       Speed seems to be an issue with the username lookup.
v0.37  added list-request for EMWAC mailing lists.
v0.38  adding "nobody@a.com" to the plugin1 config file causes all mail
       being delivered for a.com to be accepted without passing the user check
v0.39  fixed a small problem with plugin1 file parsing and corrected enumerated
       string order for command list
v0.40  All date time strings now in english format
v0.41  Changed subnet internal structure to add local black and white list.
       Eliminates the need for a local DNS server
v0.42  NT User verification changed to reading all members of a group at
       service start
v0.43  Rebuild with fix in NT User Verification
v0.44  After much wrestling with the user verification code, I finally
       got all the bugs out of it.  The API can be such a pain in the ass.
       Also added -version as a command line parameter.
}

unit ServerWinshoeSMTP;

interface

uses
  Classes,
  Winshoes,
  Dialogs,
  WinShoeMessage,
  ServerWinshoe,
  SUBNET;

const
  VersionString = '0.44';
  ProductName = 'SMTPRCV';
  RBLKillMethod = 2;

const
  SMTPCommands : array [1..14] of String=
   ('HELO', { From RFC 821 }
    'MAIL',
    'RCPT',
    'DATA',
    'SEND',
    'SOML',
    'SAML',
    'RSET',
    'VRFY',
    'EXPN',
    'HELP',
    'NOOP',
    'QUIT',
    'TURN');

  cHELO = 1;
  cMAIL = 2;
  cRCPT = 3;
  cDATA = 4;
  cSEND = 5;
  cSOML = 6;
  cSAML = 7;
  cRSET = 8;
  CVRFY = 9;
  cEXPN = 10;
  cHELP = 11;
  cNOOP = 12;
  cQuit = 13;
  cTURN = 14;


type
   EWinshoeSMTPError = class(EWinshoeException);

type
  pRecipientRec = ^tRecipientRec;
  tRecipientRec = record
    OriginalAddress : string;
    ValidatedAddress : string;
    LocalDomain : boolean;
  end;

type
  tRecipientList = class(TComponent)
  private
    { private declarations }
    fList : TList;
    function GetCount : integer;
  protected
    { protected declarations }
  public
    { public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy;  override;
  published
    { published declarations }
    property  Count : integer read GetCount;
    procedure Add(RecipientItem : tRecipientRec);
    procedure Clear;
    function  GetRecipientRec(Idx: integer):tRecipientRec;
  end;


type
  TSMTPMsg = class(TComponent)
    private
      fSendingDomain : string;
      fVerifiedSendingDomain : string;
      fSendingIP : string;
      fFromStr : string;
      fRecipients : TRecipientList;
      fData : TStringList;
      fDateTime : TDateTime;
      fMsgId : string;
      {vvvv 0.28}
      fFailReason : string;       {0.26}
      fDenyAllCommands : boolean; {0.26}
      fLocalSender : boolean;
    protected
    public
      procedure Clear(FirstTime : boolean);
      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;
    published
      property SendingDomain : string read fSendingDomain write fSendingDomain;
      property VerifiedSendingDomain : string read fVerifiedSendingDomain write fVerifiedSendingDomain;
      property SendingIP : string read fSendingIP write fSendingIP;
      property Recipients : TRecipientList read fRecipients write fRecipients;
      property FromStr : string read fFromStr write fFromStr;
      property Data : TStringList read fdata write fdata;
      property DateTime : TDateTime read FDateTime write FdateTime;
      property MsgId : string read FmsgId write fMsgId;
    end;

type
  THeloEvent= procedure(Thread: TWinshoeServerThread; Domain: string;
                        SessionMsg: tSMTPMsg; var Handled : boolean) of object;
  TMailEvent= procedure(Thread: TWinshoeServerThread; FromStr: string;
                        SessionMsg: tSMTPMsg; var Handled : boolean) of object;
  TRCPTEvent= procedure(Thread: TWinshoeServerThread; ToStr: string;
                        SessionMsg: tSMTPMsg; var Handled : boolean) of object;
  TVRFYEvent= procedure(Thread: TWinshoeServerThread; aName: string;
                        var Handled : boolean) of object;
  TEXPNEvent= procedure(Thread: TWinshoeServerThread; aListName: string;
                        var Handled : boolean) of object;
  THELPEvent= procedure(Thread: TWinshoeServerThread; ACommand: string;
                        var Handled : boolean) of object;
  TDATAEvent= procedure(Thread: TWinshoeServerThread; SessionMsg :tSMTPMsg;
                        var Handled : boolean) of object;
  TSMTPEvent= procedure(Thread: TWinshoeServerThread; var Handled : boolean)
                        of object;
  TErrorEvent= procedure(Thread: TWinshoeServerThread; ArgSTr: string;
                         var Handled : boolean) of object;
  TNextIDEvent= procedure(var ID: integer; var Handled : boolean) of object;
{ 0.42
  TNTUserEvent= function(const Name : string) : boolean of object;
}
  TPOPAuthEvent = procedure(const IPAddy : string; var Valid : boolean;
                            var Handled : boolean) of object;
  TLogFileEvent = procedure(const LogFileEntry : string) of object;{0.11}

  TWinshoeSMTPListener = class(TWinshoeListener)
  private
{other support}
    fOnCommandHELO : THeloEvent;
    fOnCommandMAIL : TMailEvent;
    fOnCommandRCPT : TRcptEvent;
    fOnCommandDATA : TDATAEvent;
    fOnCommandSEND : TMailEvent;
    fOnCommandSOML : TMailEvent;
    fOnCommandSAML : TMailEvent;
    fOnCommandRSET : TDataEvent;
    fOnCommandVRFY : TVRFYEvent;
    fOnCommandEXPN : TEXPNEvent;
    fOnCommandHELP : THelpEvent;
    fOnCommandNOOP : TSMTPEvent;
    fOnCommandQUIT : TSMTPEvent;
    fOnCommandTURN : TDATAEvent;
    fOnCommandError : TErrorEvent;
    fOnLogFileEvent : TLogFileEvent;{0.11}{0.29}
  protected
    function VerifyLocalSender(SendingIP : string) : boolean; Virtual; Abstract;
    function VerifySendingDomain(SendingIP : string) : string; Virtual; Abstract;
    function PassedBlackLists(SendingIP : string; var Failure : string) : boolean; Virtual; Abstract;

    procedure DoCommandHELO(Thread: TWinshoeServerThread; Domain: string;
                            SessionMsg: tSMTPMsg; var Handled: boolean); Virtual; Abstract;
    procedure DoCommandMAIL(Thread: TWinshoeServerThread; FromStr: string;
                            SessionMsg: tSMTPMsg; var Handled: boolean); Virtual; Abstract;
    procedure DoCommandRCPT(Thread: TWinshoeServerThread; ToStr: string;
                            SessionMsg: tSMTPMsg; var Handled: boolean); Virtual; Abstract;
    procedure DoCommandDATA(Thread: TWinshoeServerThread; SessionMsg: tSMTPMsg;
                            var Handled : boolean); Virtual;  Abstract;
    procedure DoCommandSEND(Thread: TWinshoeServerThread; FromStr: string;
                            SessionMsg: tSMTPMsg; var Handled: boolean); Virtual; Abstract;
    procedure DoCommandSOML(Thread: TWinshoeServerThread; FromStr: string;
                            SessionMsg: tSMTPMsg; var Handled : boolean);Virtual; Abstract;
    procedure DoCommandSAML(Thread: TWinshoeServerThread; FromStr: string;
                            SessionMsg: tSMTPMsg; var Handled: boolean); Virtual; Abstract;
    procedure DoCommandRSET(Thread: TWinshoeServerThread; SessionMsg: tSMTPMsg;
                            var Handled : boolean); Virtual; Abstract;
    procedure DoCommandVRFY(Thread: TWinshoeServerThread; aName: string;
                            var Handled : boolean); Virtual; Abstract;
    procedure DoCommandEXPN(Thread: TWinshoeServerThread; aListName: string;
                            var Handled : boolean); Virtual; Abstract;
    procedure DoCommandHELP(Thread: TWinshoeServerThread; aCommand: string;
                            var Handled : boolean); Virtual; Abstract;
    procedure DoCommandNOOP(Thread: TWinshoeServerThread;
                             var Handled : boolean); Virtual; Abstract;
    procedure DoCommandQUIT(Thread: TWinshoeServerThread;
                            var Handled : boolean); Virtual; Abstract;
    procedure DoCommandTURN(Thread: TWinshoeServerThread;SessionMsg: tSMTPMsg;
                            var Handled : boolean); Virtual; Abstract;
    procedure DoCommandError(Thread: TWinshoeServerThread; ArgSTr: string;
                             var Handled : boolean); Virtual; Abstract;

    function DoExecute(Thread: TWinshoeServerThread): boolean; override;
  public
    constructor Create(AOwner: TComponent); override;

  published
    property OnCommandHELO : THELOEvent read fOnCommandHELO write fOnCommandHELO;
    property OnCommandMAIL : TMAILEvent read fOnCommandMAIL write fOnCommandMAIL;
    property OnCommandRCPT : TRCPTEvent read fOnCommandRCPT write fOnCommandRCPT;
    property OnCommandDATA : TDATAEvent read fOnCommandDATA write fOnCommandDATA;
    property OnCommandSEND : TMAILEvent read fOnCommandSEND write fOnCommandSEND;
    property OnCommandSOML : TMAILEvent read fOnCommandSOML write fOnCommandSOML;
    property OnCommandSAML : TMAILEvent read fOnCommandSAML write fOnCommandSAML;
    property OnCommandRSET : TDataEvent read fOnCommandRSET write fOnCommandRSET;
    property OnCommandVRFY : TVRFYEvent read fOnCommandVRFY write fOnCommandVRFY;
    property OnCommandEXPN : TEXPNEvent read fOnCommandEXPN write fOnCommandEXPN;
    property OnCommandHELP : THELPEvent read fOnCommandHELP write fOnCommandHELP;
    property OnCommandNOOP : TSMTPEvent read fOnCommandNOOP write fOnCommandNOOP;
    property OnCommandQUIT : TSMTPEvent read fOnCommandQUIT write fOnCommandQUIT;
    property OnCommandTURN : TDATAEvent read fOnCommandTURN write fOnCommandTURN;
    property OnCommandError : TErrorEvent read fOnCommandError write fOnCommandError;
    property OnLogFileEvent : TLogFileEvent read fOnLogFileEvent write fOnLogFileEvent; {0.11}{0.29}
  end;

type
  TWinshoeSMTPServer = class(TWinshoeSMTPListener)
    private
      fLocalDomains : TStrings;
      fLocalNets : TList;
      fLocalBlackList : TList;
      fLocalBlackListNotice : string;
      fWhiteList : TList;
      fLocalUsers : TStrings;
      fSkipUserVerify : TStrings;
      fDNSLists : TStrings;
      fHeaderText : TStrings;
      fDoVerifyDomain : boolean;
      fLogDebug,
      fForwardMail,
      fCheckUser, {0.33}
      fExtendedErrorRecording,{0.35}
      fDoDNSLists : boolean;
      fMailWriteDir : string;
      fServerName : string;
      fOnNextIDEvent : TNextIDEvent;
{     fOnNTUserEvent : TNTUserEvent;}
      fonPOPAuth: TPOPAuthEvent;
      fDoPOPAuth: boolean;

    protected
      procedure SetLocalDomains( Value :Tstrings);
      procedure SetLocalUsers( Value :Tstrings);
      procedure SetSkipUserVerify( Value :Tstrings);
      procedure SetDNSLists( Value :Tstrings);
      procedure SetHeaderText( Value :Tstrings);
      procedure DoConnect(Thread: TWinshoeServerThread); override;
      procedure DoCommandHELO(Thread: TWinshoeServerThread; Domain: string;
                              SessionMsg: tSMTPMsg; var Handled: boolean); override;
      procedure DoCommandMAIL(Thread: TWinshoeServerThread; FromStr: string;
                              SessionMsg: tSMTPMsg; var Handled: boolean); override;
      procedure DoCommandRCPT(Thread: TWinshoeServerThread; ToStr: string;
                              SessionMsg: tSMTPMsg; var Handled: boolean); override;
      procedure DoCommandDATA(Thread: TWinshoeServerThread; SessionMsg: tSMTPMsg;
                              var Handled : boolean); override;
      procedure DoCommandSEND(Thread: TWinshoeServerThread; FromStr: string;
                              SessionMsg: tSMTPMsg; var Handled: boolean); override;
      procedure DoCommandSOML(Thread: TWinshoeServerThread; FromStr: string;
                              SessionMsg: tSMTPMsg; var Handled: boolean); override;
      procedure DoCommandSAML(Thread: TWinshoeServerThread; FromStr: string;
                              SessionMsg: tSMTPMsg; var Handled: boolean); override;
      procedure DoCommandRSET(Thread: TWinshoeServerThread; SessionMsg: tSMTPMsg;
                              var Handled : boolean); override;
      procedure DoCommandVRFY(Thread: TWinshoeServerThread; aName: string;
                               var Handled : boolean); override;
      procedure DoCommandEXPN(Thread: TWinshoeServerThread; aListName: string;
                               var Handled : boolean); override;
      procedure DoCommandHELP(Thread: TWinshoeServerThread; aCommand: string;
                               var Handled : boolean); override;
      procedure DoCommandNOOP(Thread: TWinshoeServerThread;
                              var Handled: boolean); override;
      procedure DoCommandQUIT(Thread: TWinshoeServerThread;
                              var Handled : boolean); override;
      procedure DoCommandTURN(Thread: TWinshoeServerThread; SessionMsg: tSMTPMsg;
                              var Handled : boolean); override;
      procedure DoCommandError(Thread: TWinshoeServerThread; ArgSTr: string;
                               var Handled : boolean); override;
      function VerifyLocalSender(SendingIP : string) : boolean; override;
      function PassedBlackLists(SendingIP : string; var Failure : string) : boolean; override;
      function VerifySendingDomain(SendingIP : string) : string; override;

      procedure WriteDebugMsg(const aStr :string);
      function IsLocalDomain(const Domain : string): boolean;
      function IsInNetList(const IP : string; fNetList : TList): boolean;
      function IsLocalUser(const Name : string): boolean;
      function GetNextID : longint;
{     function IsNTUser(const Name : string): boolean;} {0.36}{0.42}
      function DoPOPLogVerify(SendingIP : string) : boolean;
      procedure SetHeaderStrsToValues(HeaderStrs: TStringList);
      function WriteMessageToFile(SessionMsg: TSMTPMsg): boolean;
      function WriteMail(Thread : TWinshoeServerThread; SessionMsg: tSMTPMsg): boolean;
      function GetLocalFQDNName : string;
    public
      constructor Create(AOwner : TComponent); override;
      destructor Destroy; override;
    published
      property OnNextIDEvent : TNextIDEvent read fOnNextIDEvent write fOnNextIDEvent;
{     property OnNTUserEvent : TNTUserEvent read fOnNTUserEvent write fOnNTUserEvent;}
      property OnPopAuth : TPOPAuthEvent read fOnPOPAuth write fOnPOPAuth;
      property DoVerifyDomain : boolean read fDoVerifyDomain write fDoVerifyDomain;
      property LocalDomains : TStrings read FLocalDomains write SetLocalDomains;
      property LocalNets : TList read fLocalNets write fLocalNets;
      property LocalBlackList : TList read fLocalBlackList write fLocalBlackList;
      property WhiteList : TList read fWhiteList write fWhiteList;
      property LocalUsers : TStrings read FLocalUsers write SetLocalUsers;
      property SkipUserVerify : TStrings read FSkipUserVerify write SetSkipUserVerify;{0.38}
      property DNSLists : TStrings read FDNSLists write SetDNSLists;
      property HeaderText : TStrings read FHeaderText write SetHeaderText;
      property LogDebug : boolean read fLogDebug write fLogDebug;
      property DoDNSLists : boolean read fDoDNSLists write fDoDNSLists;
      property DoPOPAuth : boolean read fDoPOPAuth write fDoPOPAuth;
      property MailWriteDir : string read fMailWriteDir write fMailWriteDir;
      property LocalBlackListNotice : string read fLocalBlackListNotice write fLocalBlackListNotice;
      property ForwardMail : boolean read fForwardMail write fForwardMail;
      property CheckUser : boolean read fCheckUser write fCheckUser; {0.33}
      property ExtendedErrorRecording : boolean read fExtendedErrorRecording write fExtendedErrorRecording; {0.35}
    end;

// Procs

  procedure Register;

implementation

uses
  Windows,
  FileCtrl,
  GlobalWinshoe,
  WinsockIntf,
  SysUtils,DEFDATE;

procedure Register;
begin
  RegisterComponents('Winshoes', [TWinshoeSMTPServer]);
end;

//------------------------------------------------------------------------------
//                        Start RecipientList Code
//------------------------------------------------------------------------------

procedure SetDefaultDateTimeStrings; {0.40 in defdate}
{fix date/time strings so that they appear american format,
some mailers puke on international formats}
var
  Loop : integer;
begin
  for Loop := 1 to 12 do begin
    ShortMonthNames[Loop] := DefShortMonthNames[Loop];
    LongMonthNames[Loop] := DefLongMonthNames[Loop];
  end;
  for Loop := 1 to 7 do begin
    ShortDayNames[Loop] := DefShortDayNames[Loop];
    LongDayNames[Loop] := DefLongDayNames[Loop]
  end;
end;

constructor TRecipientList.Create(AOwner : TComponent);
begin
  Inherited Create(AOwner);
  fList := TList.Create;
  fList.Clear;
  SetDefaultDateTimeStrings;
end;

destructor TRecipientList.Destroy;
var
  Idx : integer;
  aPtr : pRecipientRec;
begin
  if fList.Count > 0 then begin
    For Idx := 0 To (fList.count -1) do begin
       aPtr := fList.Items[Idx];
       Dispose(aPtr);
     end;
   end;
  fList.Free;
  Inherited Destroy;
end;

procedure TRecipientList.Clear;
var
  aPtr : pRecipientRec;
  Idx : integer;
begin
  if fList.count > 0 then begin
    For Idx := 0 To (fList.count -1) do begin
      aPtr := fList.Items[Idx];
      Dispose(aPtr);
    end;
    fList.Clear;
  end;
end;

function TRecipientList.GetCount : integer;
begin
  GetCount := fList.Count;
end;

procedure TRecipientList.Add(RecipientItem : tRecipientRec);
var
  pRecipientItem : pRecipientRec;
begin
  New(pRecipientItem);
  pRecipientItem^ := RecipientItem;
  fList.Add(pRecipientItem);
end;

function TRecipientList.GetRecipientRec(Idx: integer):tRecipientRec;
var
  RecipientItem : tRecipientRec;
begin
  FillChar(RecipientItem,SizeOf(RecipientItem),0);
  Result := RecipientItem;
  if (Idx < fList.Count) And (Idx >= 0) then begin
    Result := tRecipientRec(fList.Items[Idx]^);
  end;
end;

//------------------------------------------------------------------------------
//                        end RecipientList Code
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
//                        Start Global Code
//------------------------------------------------------------------------------
function GetInternetDateTimeStr(aDateTime : TDateTime): string;
var
  lpTimeZoneInformation: TTimeZoneInformation;
  S,S2 : string;
  B : integer;
  Neg : boolean;
  Return : longint;
begin                           { GetInternetDateTimeStr }
  S := FormatDateTime('ddd, dd mmm yyyy hh":"nn":"ss ',aDateTime);{0.40}
  Return := GetTimeZoneInformation(lpTimeZoneInformation);
  if not (Return = -1) then begin
    B := lpTimeZoneInformation.Bias; {this is the way gettzinfo works, look it up}
    if (Return = TIME_ZONE_ID_DAYLIGHT) then
      B := B + lpTimeZoneInformation.DaylightBias; {0.34}
    B := (-B div 60) * 100;
    Neg := false;
    if (B < 0) then Neg := true;
    S2 := IntToStr(abs(B));
    while (length(S2) < 4) do S2 := '0'+S2;
    if Neg then S2 := '-'+S2;
    S := S + S2;
  end;
  Result := S;
end;                            { GetInternetDateTimeStr }

function LogFileDateTime : string;{0.11}
begin
  Result := '['+FormatDateTime('dd"/"mmm"/"yyyy":"hh":"nn":"ss',Now)+']';{0.40}
end;

function FixDirBackSlash(const aDir : string): string;
begin
  Result := aDir;
  if (length(aDir) > 0) and (AnsiLastChar(aDir)^ <> '\') then
    Result := aDir + '\'
end;

procedure StripToFrom(var AddrStr: string);
var
  UpAddrStr : string;
  IPos : integer;
begin
  UpAddrStr := UpperCase(AddrStr);
  Ipos := Pos('TO:',UpaddrStr);
  if IPos <> 0 then  Delete(AddrStr,Ipos,3)
  else begin
    Ipos := Pos('FROM:',UpAddrStr);
    if Ipos <> 0 then Delete(AddrStr,Ipos,5);
  end;
  AddrStr := Trim(AddrStr);
end;

function CharCount(S : string; C : char) : integer;
var
  Loop : integer;
  Count : integer;
begin
  Count := 0;
  for Loop := 1 to length(S) do begin
    if (S[Loop] = C) then inc(Count);
  end;
  CharCount := Count;
end;

function ParseAddress(const FullAddrStr: string;
                      var Domain, PrettyAddress, MachineAddress : string) : boolean;
{v0.16 major cleanup, now correctly deals with %,: and !}
var
  InAddr : string;
  Comment : string;
  HumanName : string;
  UserName : string;
  P,Q : integer;
  S1,S2 : string;
begin
  {get rid of comments first}
  ParseAddress := true;
  InAddr := FullAddrStr;
  StripToFrom(InAddr);
  P := pos('(',InAddr);
  Q := pos(')',InAddr);
  Comment := '';
  if (P > 0) and (Q > 0) then begin
    Comment := copy(InAddr,1,Q-1);
    delete(Comment,1,P);
    InAddr := copy(InAddr,1,P-1);
  end;
  InAddr := Trim(InAddr);

  {now parse out the machine and human address if both are listed}
  MachineAddress := InAddr;
  P := pos('<',MachineAddress);
  Q := pos('>',MachineAddress);
  if (P > 0) and (Q > 0) then begin {bingo, we got a machine address surrounded by < >}
    MachineAddress := copy(MachineAddress,1,Q-1);
    delete(MachineAddress,1,P);
    InAddr := copy(InAddr,1,P-1) + copy(InAddr,Q+1,length(InAddr));
    InAddr := trim(InAddr);
    HumanName := InAddr;
    while (pos('"',HumanName) > 0) do
      delete(HumanName,pos('"',HumanName),1);
    HumanName := Trim(HumanName);
  end else begin
    MachineAddress := InAddr;
    HumanName := '';
  end;

  {sanitation time}
  if (CharCount(MachineAddress,':') > 1) then begin
    ParseAddress := false;
    exit;
  end;
  P := pos(':',MachineAddress);
  if (P > 0) then {a:b becomes ba}
    MachineAddress := copy(MachineAddress,P+1,255) +
                      copy(MachineAddress,1,P-1);
  while (pos('"',MachineAddress) > 0) do {remove all ", they're useless}
    delete(MachineAddress,pos('"',MachineAddress),1);
  P := pos('%',MachineAddress);
  while (P > 0) do begin {replace all % with @}
    MachineAddress[P] := '@';
    P := pos('%',MachineAddress);
  end;
  P := pos('!',MachineAddress);
  while (P > 0) do begin
  {OK slightly tricky to read this, this procedure does this
   it switches a!b to b@a but only between @ symbols, IE:
   a!b becomes b@a,
   c@a!b@d becomes c@b@a@d
   a!b@d becomes b@a@d
   c@a!b becomes c@b@a}
    S1 := copy(MachineAddress,1,P-1);
    S2 := MachineAddress;
    delete(S2,1,P);
    while (pos('@',S1) > 0) do
      delete(S1,1,pos('@',S1));
    if (pos('@',S2) > 0) then
      S2 := copy(S2,1,pos('@',S2)-1);
    MachineAddress := copy(MachineAddress,1,P-length(S1)-1)+S2+'@'+S1+
                      copy(machineAddress,P+length(S2)+1,255);
    P := pos('!',MachineAddress);
  end;
  {split the username and domain}
  Domain := MachineAddress;
  P := pos('@',Domain);
  if (P > 0) then begin
    delete(Domain,1,P);
    P := pos('@',Domain);
    if (P > 0) then {ignore second @ and everything thereafter, this is a hack to
                     use multipath for relay}
      Domain := copy(Domain,1,P-1);
    P := pos('[',Domain);{strip [] from any domain, should prolly verify
                          valid IP number and replace to maintain standard}
    if (P > 0) then
      delete(Domain,P,1);
    Q := pos(']',Domain);
    if (Q > 0) then
      delete(Domain,Q,1);
    UserName := MachineAddress;
    P := pos('@',UserName);
    UserName := copy(UserName,1,P-1);
    UserName := Trim(UserName);
    if (UserName = '') then UserName := 'trashbin';{default name when none is specified}
  end else begin
    ParseAddress := false;
    UserName := Domain;
    Domain := '';
  end;
  {generate correctly formatted returns}
  MachineAddress := UserName + '@' + Domain;
  PrettyAddress := '';
  if (HumanName <> '') then begin
    PrettyAddress := '"'+HumanName+'" ';
  end;
  PrettyAddress := PrettyAddress + '<' + MachineAddress + '>';
  if (Comment <> '') then begin
    PrettyAddress := PrettyAddress + ' ('+Comment+')';
  end;
end;

//------------------------------------------------------------------------------
//                        Start SMTP Message Code
//------------------------------------------------------------------------------
constructor TSMTPMsg.Create(AOwner: TComponent);
begin                           { Create }
  Inherited Create(AOwner);
  fRecipients := TRecipientList.Create(Self);
  fData := TSTringList.Create;
end;                            { Create }

destructor TSMTPMsg.Destroy;
begin                           { Destroy }
  Fdata.Free;
  FRecipients.Free;
  Inherited Destroy;
end;                            { Destroy }

procedure TSMTPMsg.Clear(FirstTime : boolean);
begin
  fRecipients.Clear;
  fData.Clear;
  fFromStr := '';
  fDateTime := 0;
  fMsgId := '';
  if FirstTime then begin
    fSendingDomain := '';
    fVerifiedSendingDomain := '';
    fSendingIP := '255.255.255.255';
  end;
end;

//------------------------------------------------------------------------------
//  Start of TWinshoeSMTPListener Code
//------------------------------------------------------------------------------
constructor TWinshoeSMTPListener.Create(AOwner: TComponent);
begin                           { Create }
  Inherited Create(AOwner);
  Port :=  WSPORT_SMTP;;
end;                            { Create }

function TWinshoeSMTPListener.DoExecute(Thread: TWinshoeServerThread): boolean;
var
  RcvdStr,
  ArgStr,
  sCmd: string;
  CmdNum : integer;
  Handled : boolean;
  SessionMsg : tSMTPMsg;

  function GetFirstTokenDeleteFromArg(var s1: string; const sDelim: string): string;
  var
    nPos: integer;
  begin                         { GetFirstTokenDeleteFromArg }
    nPos := Pos(sDelim, s1);
    if nPos = 0 then nPos := Length(s1) + 1;
    Result := Copy(s1, 1, nPos - 1);
    Delete(s1, 1, nPos);
    S1 := Trim(S1);
  end;                          { GetFirstTokenDeleteFromArg }

begin                           { DoExecute }
  Result := Inherited DoExecute(Thread);
  if Result then exit;
  SessionMsg := tSMTPMsg.Create(Self);
  SessionMsg.Clear(true);
{  try}  
    with SessionMsg do begin
      SendingIP := Thread.Connection.PeerAddress;
      fLocalSender := VerifyLocalSender(SendingIP);
      fFailReason := '';
      if fLocalSender or PassedBlackLists(SendingIP,fFailReason) then begin
        fDenyAllCommands := false;
      end else begin
        fDenyAllCommands := true;
        if Assigned(OnLogFileEvent) then OnLogFileEvent(LogFileDateTime+' '+SendingIP+' 554 access denied, '+fFailReason);{0.11}{0.29}
      end;
      VerifiedSendingDomain := VerifySendingDomain(SendingIP);
    end;
    with Thread.Connection do begin
      While Connected do begin
        Handled := False;
        RcvdStr := ReadLn;
        ArgStr := RcvdStr;
        sCmd := UpperCase(GetFirstTokenDeleteFromArg(ArgStr,CHAR32));
        CmdNum := Succ(PosInStrArray(Uppercase(sCmd),SMTPCommands));
        case CmdNum of
          cHELO : DoCommandHELO(Thread, ArgStr, SessionMsg, Handled);
          cMAIL : DoCommandMAIL(Thread, ArgStr, SessionMsg, Handled);
          cRCPT : DoCommandRCPT(Thread, ArgStr, SessionMsg, Handled);
          cDATA : DoCommandDATA(Thread, SessionMsg, Handled);
          cSEND : DoCommandSEND(Thread, ArgStr, SessionMsg, Handled);
          cSOML : DoCommandSOML(Thread, ArgStr, SessionMsg, Handled);
          cSAML : DoCommandSAML(Thread, ArgStr, SessionMsg, Handled);
          cRSET : DoCommandRSET(Thread, SessionMsg, Handled);
          CVRFY : DoCommandVRFY(Thread, ArgStr, Handled);
          cEXPN : DoCommandEXPN(Thread, ArgStr, Handled);
          cHELP : DoCommandHELP(Thread, ArgStr, Handled);
          cNOOP : DoCommandNOOP(Thread, Handled);
          cQuit : DoCommandQUIT(Thread, Handled);
          cTURN : DoCommandTURN(Thread, SessionMsg, Handled);
          else DoCommandError(Thread, RcvdStr, Handled);
        end; {case}
      end; {while}
    end; {with}
{  finally}
    SessionMsg.Free;
{  end;}
end;                            { DoExecute }

//------------------------------------------------------------------------------
//  Start of TWinshoeSMTPServer Code
//------------------------------------------------------------------------------
function TWinshoeSMTPServer.GetLocalFQDNName : string; {0.32}
var
  aHost: PHostEnt;
begin
  aHost := WinsockInterface.gethostbyname(PChar(LocalName));
  if aHost = nil then begin
    CheckForSocketError(SOCKET_ERROR);
    Result := '';
  end else begin
    Result := String(PChar(aHost.h_name));
  end;
end;

constructor TWinshoeSMTPServer.Create(AOwner: TComponent);
var
  S : string;
begin
  Inherited Create(AOwner);
  fLocalDomains := TStringList.Create;
  fLocalNets := TList.Create;
  fWhiteList := TList.Create;
  fLocalBlackList := TList.Create;
  fLocalUsers := TStringList.Create;
  fSkipUserVerify := TStringList.Create;
  fDNSLists := TStringList.Create;
  fHeaderText := TStringList.Create;
  S := LocalAddress;
{  fServerName := ResolveIP(LocalAddress);}
  fServerName := GetLocalFQDNName; {0.32}
  if (fServerName = '') then {0.20}
    fServerName := LocalAddress;
end;

destructor TWinShoeSMTPServer.Destroy;
begin
  fLocalDomains.Free;
  fLocalNets.Free;
  fLocalBlackList.Free;
  fWhiteList.Free;
  fLocalUsers.Free;
  fSkipUserVerify.Free;
  fDNSLists.Free;
  fHeaderText.Free;
  Inherited Destroy;
end;

procedure TWinshoeSMTPServer.WriteDebugMsg(const aStr :string);
var
  aFile :Text;
begin                           { WriteDebugMsg }
  {$I-}
  if boolean(IoResult) then;
  AssignFile(aFile,FixDirBackSlash(fMailWriteDir) +'Smtp.Txt');
  Append(aFile);
  if IoResult <> 0 then ReWrite(Afile);
  WriteLn(Afile,DateTimeToStr(Now)+' '+Astr);
  Close(Afile);
  {$I+}
end;                           { WriteDebugMsg }

procedure TWinshoeSMTPServer.SetLocalDomains(Value: TStrings);
begin
  FLocalDomains.Assign(Value);
end;

procedure TWinshoeSMTPServer.SetLocalUsers(Value: TStrings);
begin
  FLocalUsers.Assign(Value);
end;

procedure TWinshoeSMTPServer.SetSkipUserVerify(Value: TStrings);
begin
  FSkipUserVerify.Assign(Value);
end;

procedure TWinshoeSMTPServer.SetDNSLists(Value: TStrings);
begin
  FDNSLists.Assign(Value);
end;

procedure TWinshoeSMTPServer.SetHeaderText(Value: TStrings);
begin
  FHeaderText.Assign(Value);
end;


function TWinShoeSMTPServer.IsLocalDomain(const Domain : string): boolean;
var
  ActualDomain : string;
  idx : integer;
begin
  IsLocalDomain := false;
  ActualDomain := Trim(UpperCase(Domain));
  if fLocalDomains.Count > 0 then begin
    For idx := 0 To fLocalDomains.Count - 1 do begin
      if CompareText(ActualDomain,Trim(UpperCase(fLocalDomains.Strings[Idx]))) = 0 then begin
        if fLogDebug then WriteDebugMsg('IsLocalDomain says LocalDomain Is True');
        Result := True;
        Exit;
      end;
    end;
  end;
end;

function TWinShoeSMTPServer.IsInNetList(const IP : string; fNetList : TList): boolean;
var
  ActualIP : string;
  idx : integer;
  TempNet : PSubNet;
  IPAddy : longword;
begin
  IsInNetList := false;
  ActualIP := Trim(IP);
  IPAddy := ParseIPAddress(ActualIP);
  if fNetList.Count > 0 then begin
    For idx := 0 To fNetList.Count - 1 do begin
      TempNet := fNetList[Idx];
      IsInNetList := Result or ((IPAddy and TempNet.Mask) = TempNet.Net);
    end;
  end;
end;

function TWinShoeSMTPServer.IsLocalUser(const Name : string): boolean; {0.33}
var
  Domain,ActualName : string;
  P,idx : integer;
begin
  IsLocalUser := false;
  ActualName := UpperCase(Trim(Name));
  P := pos('@',ActualName);
  Domain := '';
  if (P > 0) then begin
    Domain := ActualName;
    delete(Domain,1,P);
    if (fSkipUserVerify.Count > 0) then begin {0.38}
      For idx := 0 To (fSkipUserVerify.Count - 1) do begin
        if (Domain = UpperCase(Trim(fSkipUserVerify.Strings[Idx]))) then begin
          IsLocalUser := true;
          exit;
        end;
      end;
    end;
    ActualName := copy(ActualName,1,P-1);
  end;
  if fLocalUsers.Count > 0 then begin
    For idx := 0 To fLocalUsers.Count - 1 do begin
      IsLocalUser := Result or (ActualName = UpperCase(Trim(fLocalUsers.Strings[Idx])));
    end;
  end;
{ 0.42
  if not Result then begin
    Result := IsNTUser(ActualName);
  end;
}
end;

procedure TWinShoeSMTPServer.DoConnect(Thread: TWinshoeServerThread);
begin                           { DoConnect }
{ I HAD to remove this from here at 0.28, because it's not thread specific
  This caused 2 concurrent hits to allow spam to get through

  IP := Thread.Connection.PeerAddress;
  fLocalSender := VerifyLocalSender(IP);
  fFailReason := '';
  if fLocalSender or PassedBlackLists(IP,fFailReason) then begin
    fDenyAllCommands := false;
}
    Thread.Connection.WriteLn('220 '+fServerName +' '+ProductName+' SMTP Receiver Version '+VersionString+' Ready');
    if fLogDebug  then begin
      WriteDebugMsg('');
      WriteDeBugMsg('');
      WriteDebugMsg('Connect says 220 '+fServerName +' '+ProductName+' SMTP Receiver Service '+VersionString+' Ready');
    end;
(*
  end else begin
    try
      fDenyAllCommands := true; {0.26}
      if Assigned(OnLogFileEvent) then OnLogFileEvent(LogFileDateTime+' '+IP+' 554 access denied, '+fFailReason);{0.11}
      if (RBLKillMethod = 1) then begin
        Thread.Connection.WriteLn('421 access denied, '+fFailReason);
        if fLogDebug then WriteDebugMsg('Dump says 421 access denied, '+fFailReason);
      end else begin
        Thread.Connection.WriteLn('220 '+fServerName +' '+ProductName+' SMTP Receiver Version '+VersionString+' Ready, access restricted');
        if fLogDebug  then begin
          WriteDebugMsg('');
          WriteDeBugMsg('');
          WriteDebugMsg('Connect says 220 '+fServerName +' '+ProductName+' SMTP Receiver Service '+VersionString+' Ready, access restricted');
        end;
      end;
    finally
      if (RBLKillMethod = 1) then
        Thread.Connection.Disconnect;
    end;
  end;
*)
end;                            { DoConnect }

procedure TWinShoeSMTPServer.DoCommandHELO(Thread: TWinshoeServerThread; Domain: string;
                                           SessionMsg: tSMTPMsg;var Handled : boolean);
var
  S : string;
begin                           { DoCommandHELO }
  if Assigned(OnCommandHELO) then OnCommandHELO(Thread, Domain,SessionMsg, Handled);
  if not Handled then begin
    with SessionMsg do begin
      if Length(SendingDomain) > 0 then begin
        Thread.Connection.WriteLn('503 HELO Command is out of sequence');
        if fLogDebug  then WriteDebugMsg('HELO says 503 Bad Command Sequence');
      end
      else begin
        SendingDomain := Domain;
        S := '';
        if SessionMsg.fDenyAllCommands then
          S := ', access restricted';
        Thread.Connection.WriteLn('250 OK '+Domain+' ('+VerifiedSendingDomain+' ['+SendingIP+'])'+S);
        if fLogDebug  then WriteDebugMsg('HELO says 250 OK '+Domain+' ('+VerifiedSendingDomain+' ['+SendingIP+'])'+S);
      end;
    end;
  end;
end;                            { DoCommandHELO }

procedure TWinShoeSMTPServer.DoCommandRCPT(Thread: TWinshoeServerThread; ToStr: string;
                                           SessionMsg: tSMTPMsg; var Handled : boolean);
var
  aRecipient : TRecipientRec;
  Domain,PrettyAddress,MachineAddress : string;
begin                           { DoCommandRCPT }
  if Assigned(OnCommandRCPT) then OnCommandRCPT(Thread, ToStr,SessionMsg, Handled);
  if not Handled then begin
    if ParseAddress(ToStr,Domain,PrettyAddress,MachineAddress) then begin
      FillChar(ARecipient,SizeOf(ARecipient),0);
      with aRecipient do begin
        OriginalAddress := ToStr;
        ValidatedAddress := MachineAddress;
        LocalDomain := IsLocalDomain(Domain);
      end;
      if (not SessionMsg.fDenyAllCommands) then begin
        if aRecipient.LocalDomain then begin
          if ((not fCheckUser) or IsLocalUser(MachineAddress)) then begin {0.33}
            Thread.Connection.WriteLn('250 OK '+PrettyAddress);
            if fLogDebug  then  WriteDebugMsg('RCPT says 250 OK '+PrettyAddress);
            SessionMsg.Recipients.Add(aRecipient);
          end else begin
            Thread.Connection.WriteLn('550 <'+MachineAddress+'>... user not valid');
            if fLogDebug  then  WriteDebugMsg('RCPT says 550 <'+MachineAddress+'>... user not valid');
            if fExtendedErrorRecording and Assigned(OnLogFileEvent) then
              OnLogFileEvent(LogFileDateTime+' '+SessionMsg.SendingIP+' 550 <'+MachineAddress+'> user not valid');{0.35}
          end;
        end else if SessionMsg.fLocalSender or fForwardMail then begin {0.21}
          Thread.Connection.WriteLn('250 OK '+PrettyAddress);
          if fLogDebug  then  WriteDebugMsg('RCPT says 250 OK '+PrettyAddress);
          SessionMsg.Recipients.Add(aRecipient);
        end else begin
          Thread.Connection.WriteLn('550 <'+MachineAddress+'>... Relaying denied');
          if fLogDebug  then  WriteDebugMsg('RCPT says 550 <'+MachineAddress+'>... Relaying denied');
          if fExtendedErrorRecording and Assigned(OnLogFileEvent) then
            OnLogFileEvent(LogFileDateTime+' '+SessionMsg.SendingIP+' 550 <'+MachineAddress+'> relaying denied');{0.35}
        end
      end else begin
        Thread.Connection.WriteLn('554 access denied, '+SessionMsg.fFailReason);
        if fLogDebug then WriteDebugMsg('Dump says 554 access denied, '+SessionMsg.fFailReason);
        if (RBLKillMethod = 3) then
          Thread.Connection.Disconnect;
      end;
    end else begin
      Thread.Connection.WriteLn('501 Syntax Error in RCPT '+TosTr);
      if fLogDebug  then  WriteDebugMsg('RCPT says 501 syntax error '+ToStr);
    end;
  end;
end;                            { DoCommandRCPT }

procedure TWinShoeSMTPServer.DoCommandMAIL(Thread: TWinshoeServerThread; FromStr: string;
                                           SessionMsg: tSMTPMsg;var Handled : boolean);
var
  Domain,PrettyAddress,MachineAddress : string;
begin                           { DoCommandMAIL }
  if Assigned(OnCommandMAIL) then OnCommandMAIL(Thread, FromStr,SessionMsg, Handled);
  if not Handled then begin
    if ParseAddress(FromStr,Domain,PrettyAddress,MachineAddress) then begin
      SessionMsg.FromStr := MachineAddress;
      Thread.Connection.WriteLn('250 OK '+PrettyAddress);
      if fLogDebug  then  WriteDebugMsg('MAIL says 250 OK '+PrettyAddress);
    end else begin
      StripToFrom(FromStr);
      SessionMsg.FromStr := FromStr;
      Thread.Connection.WriteLn('250 OK '+FromStr);
      if fLogDebug  then  WriteDebugMsg('MAIL says 250 OK '+FromStr);
{  RFC 2505 violation v0.18
      Thread.Connection.WriteLn('501 Bad From Address '+ FromStr);
      if fLogDebug  then  WriteDebugMsg('MAIL says 501 Bad From Address '+ FromStr);
}
    end;
  end;
end;                            { DoCommandMAIL }

procedure TWinShoeSMTPServer.DoCommandRSET(Thread: TWinshoeServerThread;
                                            SessionMsg: tSMTPMsg; var Handled : boolean);
begin                           { DoCommandRSET }
  if Assigned(OnCommandRSET) then OnCommandRSET(Thread, SessionMsg, Handled);
  if not Handled then begin
    SessionMsg.Clear(false);
    Thread.Connection.WriteLn('250 OK Reset Completed');
    if fLogDebug  then  WriteDebugMsg('RSET says 250 OK');
  end;
end;                            { DoCommandRSET }

procedure TWinShoeSMTPServer.DoCommandVRFY(Thread: TWinshoeServerThread;
                                           aName: string; var Handled : boolean);
begin                           { DoCommandVRFY }
  if Assigned(OnCommandVRFY) then OnCommandVRFY(Thread,aName, Handled);
  if not Handled then begin
    Thread.Connection.WriteLn('250 OK '+aname);
    if fLogDebug  then WriteDebugMsg('VRFY says 250 OK '+aname);
  end;
end;                            { DoCommandVRFY }

procedure TWinShoeSMTPServer.DoCommandNOOP(Thread: TWinshoeServerThread;
                                            var Handled : boolean);
begin                           { DoCommandNOOP }
  if Assigned(OnCommandNOOP) then OnCommandNOOP(Thread, Handled);
  if not Handled then begin
    Thread.Connection.WriteLn('250 NOOP Performed');
    if fLogDebug  then  WriteDebugMsg('HELP says 250 NOOP Performed');
  end;
end;                            { DoCommandNOOP }

procedure TWinShoeSMTPServer.DoCommandEXPN(Thread: TWinshoeServerThread;
                                            aListName: string; var Handled : boolean);
begin                           { DoCommandEXPN }
 if Assigned(OnCommandEXPN) then OnCommandEXPN(Thread,aListName, Handled);
  if not Handled then begin
    Thread.Connection.WriteLn('550 EXPN Access Denied');
    if fLogDebug  then  WriteDebugMsg('EXPN says 550 Access Denied');
  end;
end;                            { DoCommandEXPN }

procedure TWinShoeSMTPServer.DoCommandError(Thread: TWinshoeServerThread;
                                             ArgStr: string; var Handled : boolean);
begin                           { DoCommandError }
  if Assigned(OnCommandError) then OnCommandError(Thread, ArgStr, Handled);
  if not Handled then begin
    Thread.Connection.WriteLn('500 Syntax Error, Command Unrecognized '+ArgStr);
    if fLogDebug then WriteDebugMsg('Command Error says 500 Syntax error, command Unrecognized '+ArgStr);
  end;
end;                            { DoCommandError }

procedure TWinShoeSMTPServer.DoCommandQUIT(Thread: TWinshoeServerThread;
                                            var Handled : boolean);
begin                          { DoCommandQUIT }
  try
    if Assigned(OnCommandQUIT) then OnCommandQUIT(Thread, Handled);
    if not Handled then begin
      Thread.Connection.WriteLn('221 '+fServerName
                              +' closing');
     if fLogDebug then WriteDebugMsg('QUIT says 221 '+fServerName+' closing');
    end;
  finally
    Thread.Connection.Disconnect;
  end;
end;                            { DoCommandQUIT }

procedure TWinShoeSMTPServer.DoCommandSEND(Thread: TWinshoeServerThread;FromStr: string;
                                            SessionMsg: tSMTPMsg; var Handled : boolean);
begin                           { DoCommandSEND }
  if Assigned(OnCommandSEND) then OnCommandSEND(Thread, FromStr,SessionMsg, Handled);
  if not Handled then begin
    Thread.Connection.WriteLn('502 SEND messaging prohibited.');
    if fLogDebug  then  WriteDebugMsg('SEND says 502 messaging prohibited.');
  end;
end;                            { DoCommandSEND }

procedure TWinShoeSMTPServer.DoCommandSOML(Thread: TWinshoeServerThread; FromStr: string;
                                            SessionMsg: tSMTPMsg; var Handled : boolean);
begin                           { DoCommandSOML }
  if Assigned(OnCommandSOML) then OnCommandSOML(Thread, FromStr,SessionMsg, Handled);
  if not Handled then begin
    Thread.Connection.WriteLn('502 SOML messaging prohibited.');
    if fLogDebug  then  WriteDebugMsg('SOML says 502 messaging prohibited.');
  end;
end;                            { DoCommandSOML }

procedure TWinShoeSMTPServer.DoCommandSAML(Thread: TWinshoeServerThread; FromStr: string;
                                            SessionMsg: tSMTPMsg;var Handled : boolean);
begin                           { DoCommandSAML }
  if Assigned(OnCommandSAML) then OnCommandSAML(Thread,FromStr,SessionMsg, Handled);
  if not Handled then begin
    Thread.Connection.WriteLn('502 SAML messaging prohibited.');
    if fLogDebug  then  WriteDebugMsg('SAML says 502 messaging prohibited.');
  end;
end;                            { DoCommandSAML }

procedure TWinShoeSMTPServer.DoCommandHELP(Thread: TWinshoeServerThread;
                                            aCommand: string; var Handled : boolean);
begin                           { DoCommandHELP }
  if Assigned(OnCommandHELP) then OnCommandHELP(Thread,aCommand, Handled);
  if not Handled then begin
    Thread.Connection.WriteLn('502 HELP no help available.');
    if fLogDebug  then  WriteDebugMsg('HELP says 502 no help available.');
  end;
end;                            { DoCommandHELP }

procedure TWinShoeSMTPServer.DoCommandTURN(Thread: TWinshoeServerThread;
                                            SessionMsg: tSMTPMsg;var Handled : boolean);
begin                           { DoCommandTURN }
  if Assigned(OnCommandTURN) then OnCommandTURN(Thread, SessionMsg, Handled);
  if not Handled then begin
   Thread.Connection.WriteLn('502 TURN prohibited.');
   if fLogDebug then WriteDebugMsg('TURN Says 502  prohibited.');
  end;
end;                            { DoCommandTURN }

procedure TWinShoeSMTPServer.DoCommandDATA(Thread: TWinshoeServerThread;
                                            SessionMsg: tSMTPMsg;var Handled : boolean);
var
  RcvdStr : string;
  Finished : boolean;
begin                           { DoCommandData }
  if Assigned(OnCommandDATA) then OnCommandDATA(Thread,SessionMsg, Handled);
  if not Handled then begin
    with Thread.Connection do begin
      if SessionMsg.fDenyAllCommands then begin {0.26}
        Thread.Connection.WriteLn('554 access denied, '+SessionMsg.fFailReason);
        if fLogDebug then WriteDebugMsg('DATA says 554 access denied, '+SessionMsg.fFailReason);
      end else if (SessionMsg.Recipients.Count < 1) then begin
        Thread.Connection.WriteLn('503 No valid recipients specified');
        if fLogDebug then WriteDebugMsg('DATA says 503 No valid recipients specified');
      end else begin
        WriteLn('354 Ready for data');
        if fLogDebug  then  WriteDebugMsg('DATA says 354 Ready for data');
        // (!!0.01} Modification to handle  extra Period and blank line bug
        // Modification by Roman Evstyunichev... thanks.
        Finished := false;
        repeat
          RcvdStr := ReadLn;
          if fLogDebug  then  WriteDebugMsg('DATA says '+ RcvdStr );
         {v0.13-0.15 undoes !!0.01}
         {mod here is for SMTPDS compatibility... it doesn't remove it. MVM 013101}
          Finished := ((Length(RcvdStr) = 1) and (RcvdStr[1] = '.'));
          SessionMsg.fData.add(RcvdStr);
        until Finished;
        if WriteMail(Thread, SessionMsg) then begin
          WriteLn('250 Message received OK');
          if fLogDebug  then  WriteDebugMsg('DATA says 250 Message received OK');
        end else begin
          WriteLn('451 ERR Requested Action Aborted');
          if fLogDebug then WriteDebugMsg('DATA says 451  ERR Requested Action Aborted');
        end;
        SessionMsg.Clear(false); {0.31}
      end;
    end;
  end;
end;                            { DoCommandData }

procedure TWinshoeSMTPServer.SetHeaderStrsToValues(HeaderStrs: TStringList);
var
  Idx : integer;
begin                       { SetHeaderStrsToValues }
  For Idx := 0 To HeaderStrs.Count -1 do begin
     HeaderStrs[Idx] := StringReplace(HeaderStrs[Idx], ': ', '=', []);
  end;                      { SetHeaderStrsToValues }
end;

function TWinShoeSMTPServer.WriteMessageToFile(SessionMsg: TSMTPMsg): boolean;
var
  Recp : tRecipientRec;
  FiName ,S,aStr : string;
  f: Text;
  mIdx : integer;
  DateFound,IDFound : boolean;
  LogFileEntry : string;
begin
  Result := true;
  LogFileEntry := '';
  with SessionMsg do begin
    if (SessionMsg.Recipients.Count < 1) then begin
      Result := false;
      exit;
    end;
    DateFound := false;
    IDFound := false;
    For mIdx := 0 To Fdata.count - 1 do begin
      S := UpperCase(Trim(Fdata.Strings[mIdx]));
      if (pos('DATE',S) = 1) then DateFound := true;
      if (pos('RECEIVED',S) = 1) then DateFound := true; {0.24}
      if (pos('MESSAGE-ID',S) = 1) then IDFound := true;
      if (S = '') then break; {end of header}
    end;
    try
      try
        FiName := FixDirBackSlash(fMailWriteDir)+ChangeFileExt(MsgID,'.MSG');
        if fLogDebug then WriteDeBugMsg('WriteMsgToFile says finame is '+FiName);
        {fdata is of type tstringlist, use windows call to write to file 0.22}
        AssignFile(f,FiName);
        ReWrite(F);
        Astr := 'Received: from '+ SendingDomain +' ('+ VerifiedSendingDomain+
                ' ['+SendingIP+']) by '+fServerName;
        fData.Insert(0,Astr);
        Astr := ' ('+ProductName+' '+ VersionString + ') with SMTP id <'+MsgId+'@'+fServerName+'>;';
        fData.Insert(1,Astr);
        Astr := ' '+GetInternetDateTimeStr(Now);
        fData.Insert(2,Astr);
        if not IDFound then begin
          S := 'Message-ID: <'+MsgId+'@'+fServerName+'>';
          fData.Insert(3,S);
        end;
        if not DateFound then begin
          Astr := 'Date:'+AStr;
          fData.Insert(3,Astr);
        end;
{        fData.SaveToFile(FiName);}
        For mIdx := 0 To Fdata.count - 1 do begin
          WriteLn(F,Fdata.Strings[mIdx]);
        end;
      Except
        On Exception do
          Result := false;
        end;
    finally
      CloseFile(f);
    end;
    try
      try
        LogFileEntry := '<'+MsgID+'@'+fServerName+'> '+
                        LogFileDateTime+' '+SendingIP+' '+
                        SendingDomain+' '+FromStr;{0.11}
        FiName := FixDirBackSlash(fMailWriteDir)+ChangeFileExt(MsgID,'.RCP');
        if fLogDebug then WriteDeBugMsg('WriteMsgToFile says finame is '+FiName);
        AssignFile(f,FiName);
        ReWrite(F);
        Astr := 'Message-ID: <'+MsgID+'@'+fServerName+'>';
        WriteLn(F,Astr);
        Astr := 'Return-path: '+FromStr;
        WriteLn(F,Astr);
        For mIdx := 0 To Recipients.count - 1 do begin
          Recp := Recipients.GetRecipientRec(mIdx);
          Astr := 'Recipient: '+Recp.ValidatedAddress;
          LogFileEntry := LogFileEntry + ' '+Recp.ValidatedAddress;
          WriteLn(F,Astr);
        end;
        if Assigned(OnLogFileEvent) then OnLogFileEvent(LogFileEntry);{0.11}
      Except
        On Exception do
          Result := false;
        end;
    finally
      CloseFile(f);
    end;
  end;
end;


function TWinShoeSMTPServer.GetNextID : longint;
var
  ID : longint;
  Handled : boolean;
begin
  if Assigned(OnNextIDEvent) then OnNextIDEvent(ID , Handled);
  if not Handled then
    ID := 1;
  GetNextID := ID;
end;

(*  0.42
function TWinShoeSMTPServer.IsNTUser(const Name : string): boolean; {0.36}
begin
  if Assigned(OnNTUserEvent) then
    Result := OnNTUserEvent(Name)
  else
    Result := false;
end;
*)

function TWinShoeSMTPServer.WriteMail(Thread: TWinshoeServerThread;
                                      SessionMsg:tSMTPMsg) : boolean;
var
  S : string;
begin
  S := IntToStr(GetNextID);
  while (length(S) < 10) do
    S := '0' + S;
  SessionMsg.MsgID := 'B'+S;
  WriteMail := WriteMessageToFile(SessionMsg);
end;

function TWinShoeSMTPServer.VerifySendingDomain(SendingIP : string) : string;
begin
  VerifySendingDomain := 'unverified';
  if DoVerifyDomain then
    VerifySendingDomain := Trim(TWinShoe.ResolveIP(SendingIP));
  if (Result = '') then {0.19}
    VerifySendingDomain := 'unverified';
end;

function TWinShoeSMTPServer.PassedBlackLists(SendingIP : string;
                                             var Failure : string) : boolean;
var
  Loop : integer;
  Reverse : string;
  IPos : integer;
  Query : string;
  S : string;
  aHost: PHostEnt;
  OrgIP : string;
begin
  PassedBlackLists := true;
  if IsInNetList(SendingIP,fWhiteList) then exit; {0.41}
  if IsInNetList(SendingIP,fLocalBlackList) then begin {0.41}
    Failure := fLocalBlackListNotice;
    PassedBlackLists := false;
    exit;
  end;
  if not fDoDNSLists then exit;
  OrgIP := SendingIP;
  Reverse := '';
  IPos := pos('.',SendingIP);
  while (IPos>0) do begin
    Reverse := '.'+copy(SendingIP,1,IPos-1)+Reverse;
    delete(SendingIP,1,IPos);
    IPos := pos('.',SendingIP);
  end;
  Reverse := SendingIP+Reverse;
  for Loop := 0 to DNSLists.Count-1 do begin
    Query := Reverse + '.' + DNSLists[Loop]+#0+#0;
    aHost := WinsockInterface.GetHostByName(PChar(Query));
    if (aHost <> nil) then begin
      if (Loop < HeaderText.Count) then begin
        S := HeaderText[Loop];
        IPos := pos('%s',S);
        if (IPos > 0) then begin
          delete(S,Ipos,2);
          insert(OrgIP,S,IPos);
        end;
        Failure := S;
      end else begin
        Failure := 'see http://'+DNSLists[Loop];
      end;
      PassedBlackLists := false;
      exit;
    end;
  end;
end;

function TWinShoeSMTPServer.DoPOPLogVerify(SendingIP : string) : boolean;
var
  Local, Handled : boolean;
begin
  Result := false;
  if not DoPOPAuth then exit;
  if Assigned(OnPOPAuth) then OnPOPAuth(SendingIP, Local , Handled);
  if Handled then
    Result := Local;
end;

function TWinShoeSMTPServer.VerifyLocalSender(SendingIP : string) : boolean;
begin
  VerifyLocalSender := false;
  if IsInNetList(SendingIP,fLocalNets) then begin
    VerifyLocalSender := true;
  end else begin
    VerifyLocalSender := DoPOPLogVerify(SendingIP);
  end;
end;

end.
