{ Unit:      WinSvc
  translated from winsvc.h

 Translator: Peter Sawatzki (ps)
             Buchenhof 3, 58091 Hagen, Germany
 CompuServe: 100031,3002

  Date:    Author:
  01/03/96 ps     first version for Delphi 2.0

  Copyright (c) 1996 Peter Sawatzki. All Rights Reserved.

}

Unit WinSvc;
Interface
Uses
  Windows,
  WinNT;
Const
  SERVICES_ACTIVE_DATABASE = 'ServicesActive';
  SERVICES_FAILED_DATABASE = 'ServicesFailed';
  SC_GROUP_IDENTIFIER = '+';

  {- Value to indicate no change to an optional parameter}
  SERVICE_NO_CHANGE = $ffffffff;

  {- Service State -- for Enum Requests (Bit Mask)}
  SERVICE_ACTIVE    = $00000001;
  SERVICE_INACTIVE  = $00000002;
  SERVICE_STATE_ALL = SERVICE_ACTIVE Or SERVICE_INACTIVE;

  {- Controls}
  SERVICE_CONTROL_STOP             =  $00000001;
  SERVICE_CONTROL_PAUSE            =  $00000002;
  SERVICE_CONTROL_CONTINUE         =  $00000003;
  SERVICE_CONTROL_INTERROGATE      =  $00000004;
  SERVICE_CONTROL_SHUTDOWN         =  $00000005;

{ Service State -- for CurrentState }

  SERVICE_STOPPED                  = $00000001;
  SERVICE_START_PENDING            = $00000002;
  SERVICE_STOP_PENDING             = $00000003;
  SERVICE_RUNNING                  = $00000004;
  SERVICE_CONTINUE_PENDING         = $00000005;
  SERVICE_PAUSE_PENDING            = $00000006;
  SERVICE_PAUSED                   = $00000007;

{ Controls Accepted  (Bit Mask) }
  SERVICE_ACCEPT_STOP              = $00000001;
  SERVICE_ACCEPT_PAUSE_CONTINUE    = $00000002;
  SERVICE_ACCEPT_SHUTDOWN          = $00000004;

{ Service Control Manager object specific access types }
  SC_MANAGER_CONNECT               = $0001;
  SC_MANAGER_CREATE_SERVICE        = $0002;
  SC_MANAGER_ENUMERATE_SERVICE     = $0004;
  SC_MANAGER_LOCK                  = $0008;
  SC_MANAGER_QUERY_LOCK_STATUS     = $0010;
  SC_MANAGER_MODIFY_BOOT_CONFIG    = $0020;

  SC_MANAGER_ALL_ACCESS            = STANDARD_RIGHTS_REQUIRED
                                  Or SC_MANAGER_CONNECT
                                  Or SC_MANAGER_CREATE_SERVICE
                                  Or SC_MANAGER_ENUMERATE_SERVICE
                                  Or SC_MANAGER_LOCK
                                  Or SC_MANAGER_QUERY_LOCK_STATUS
                                  Or SC_MANAGER_MODIFY_BOOT_CONFIG;

{ Service object specific access type }
  SERVICE_QUERY_CONFIG             = $0001;
  SERVICE_CHANGE_CONFIG            = $0002;
  SERVICE_QUERY_STATUS             = $0004;
  SERVICE_ENUMERATE_DEPENDENTS     = $0008;
  SERVICE_START                    = $0010;
  SERVICE_STOP                     = $0020;
  SERVICE_PAUSE_CONTINUE           = $0040;
  SERVICE_INTERROGATE              = $0080;
  SERVICE_USER_DEFINED_CONTROL     = $0100;

  SERVICE_ALL_ACCESS               = STANDARD_RIGHTS_REQUIRED
                                  Or SERVICE_QUERY_CONFIG
                                  Or SERVICE_CHANGE_CONFIG
                                  Or SERVICE_QUERY_STATUS
                                  Or SERVICE_ENUMERATE_DEPENDENTS
                                  Or SERVICE_START
                                  Or SERVICE_STOP
                                  Or SERVICE_PAUSE_CONTINUE
                                  Or SERVICE_INTERROGATE
                                  Or SERVICE_USER_DEFINED_CONTROL;

Type
{ Handle Types }
  TSC_HANDLE = THandle;
  TSERVICE_STATUS_HANDLE = Integer;

{ Service Status Structure }

  TSERVICE_STATUS = Record
    dwServiceType,
    dwCurrentState,
    dwControlsAccepted,
    dwWin32ExitCode,
    dwServiceSpecificExitCode,
    dwCheckPoint,
    dwWaitHint: Integer
  End;

{ Service Status Enumeration Structure }

  TENUM_SERVICE_STATUSA = Record
    lpServiceName,
    lpDisplayName: PAnsiChar;
    ServiceStatus: TSERVICE_STATUS
  End;

  TENUM_SERVICE_STATUSW = Record
    lpServiceName,
    lpDisplayName: PWideChar;
    ServiceStatus: TSERVICE_STATUS
  End;

  TENUM_SERVICE_STATUS = TENUM_SERVICE_STATUSA;

{ Structures for the Lock API functions }

  TSC_LOCK = Pointer;

  TQUERY_SERVICE_LOCK_STATUSA = Record
    fIsLocked: Integer;
    lpLockOwner: PAnsiChar;
    dwLockDuration: Integer
  End;

  TQUERY_SERVICE_LOCK_STATUSW = Record
    fIsLocked: Integer;
    lpLockOwner: PWideChar;
    dwLockDuration: Integer
  End;

  TQUERY_SERVICE_LOCK_STATUS = TQUERY_SERVICE_LOCK_STATUSA;

{ Query Service Configuration Structure }

  TQUERY_SERVICE_CONFIGA = Record
    dwServiceType,
    dwStartType,
    dwErrorControl: Integer;
    lpBinaryPathName,
    lpLoadOrderGroup: PAnsiChar;
    dwTagId: Integer;
    lpDependencies,
    lpServiceStartName,
    lpDisplayName: PAnsiChar;
  End;

  TQUERY_SERVICE_CONFIGW = Record
    dwServiceType,
    dwStartType,
    dwErrorControl: Integer;
    lpBinaryPathName,
    lpLoadOrderGroup: PWideChar;
    dwTagId: Integer;
    lpDependencies,
    lpServiceStartName,
    lpDisplayName: PWideChar;
  End;

  TQUERY_SERVICE_CONFIG = TQUERY_SERVICE_CONFIGA;

{ Function Prototype for the Service Main Function }

  TService_Main_FunctionW = Procedure (dwNumServicesArgs: DWord; Var ArgVectors: PWideChar) StdCall;
  TService_Main_FunctionA = Procedure (dwNumServicesArgs: DWord; Var ArgVectors: PAnsiChar) StdCall;
  TService_Main_Function = Procedure (dwNumServicesArgs: DWord; Var ArgVectors: PChar) StdCall;

{ Service Start Table }

  TSERVICE_TABLE_ENTRYA = Record
    lpServiceName: PAnsiChar;
    lpServiceProc: TSERVICE_MAIN_FUNCTIONA
  End;

  TSERVICE_TABLE_ENTRYW = Record
    lpServiceName: PWideChar;
    lpServiceProc: TSERVICE_MAIN_FUNCTIONW
  End;

  TSERVICE_TABLE_ENTRY = TSERVICE_TABLE_ENTRYA;

{ Prototype for the Service Control Handler Function }

  THANDLER_FUNCTION = Procedure (dwControl: DWord) StdCall;

  Function ChangeServiceConfigA (hService: TSC_HANDLE; dwServiceType, dwStartType, dwErrorControl: Integer;
                                 BinaryPathName, LoadOrderGroup: PAnsiChar; Var lpdwTagId: Integer;
                                 Dependencies, ServiceStartName, Password, DisplayName: PAnsiChar): Bool; StdCall;
  Function ChangeServiceConfigW (hService: TSC_HANDLE; dwServiceType, dwStartType, dwErrorControl: Integer;
                                 BinaryPathName, LoadOrderGroup: PWideChar; Var lpdwTagId: Integer;
                                 Dependencies, ServiceStartName, Password, DisplayName: PWideChar): Bool; StdCall;
  Function ChangeServiceConfig  (hService: TSC_HANDLE; dwServiceType, dwStartType, dwErrorControl: Integer;
                                 BinaryPathName, LoadOrderGroup: PChar; Var lpdwTagId: Integer;
                                 Dependencies, ServiceStartName, Password, DisplayName: PChar): Bool; StdCall;
  Function CloseServiceHandle (hSCObject: TSC_HANDLE): Bool; StdCall;
  Function ControlService (hService: TSC_HANDLE; dwControl: Integer; Var ServiceStatus: TSERVICE_STATUS): Bool; StdCall;
  Function CreateServiceA (hSCManager: TSC_HANDLE; ServiceName, DisplayName: PAnsiChar;
                           dwDesiredAccess, dwServiceType,dwStartType,dwErrorControl: Integer;
                           BinaryPathName, lpLoadOrderGroup: PAnsiChar;
                           lpTagId: PInteger;
                           Dependencies, ServiceStartName, Password: PAnsiChar): TSC_Handle; StdCall;
  Function CreateServiceW (hSCManager: TSC_HANDLE; ServiceName, DisplayName: PWideChar;
                           dwDesiredAccess, dwServiceType,dwStartType,dwErrorControl: Integer;
                           BinaryPathName, lpLoadOrderGroup: PWideChar;
                           lpTagId: PInteger;
                           Dependencies, ServiceStartName, Password: PWideChar): TSC_Handle; StdCall;
  Function CreateService  (hSCManager: TSC_HANDLE; ServiceName, DisplayName: PChar;
                           dwDesiredAccess, dwServiceType,dwStartType,dwErrorControl: Integer;
                           BinaryPathName, lpLoadOrderGroup: PChar;
                           lpTagId: PInteger;
                           Dependencies, ServiceStartName, Password: PChar): TSC_Handle; StdCall;
  Function DeleteService (hService: TSC_HANDLE): Bool; StdCall;
  Function EnumDependentServicesA (hService: TSC_HANDLE; ServiceState: Integer;
                               Var Services: TENUM_SERVICE_STATUSA;
                                   cbBufSize: Integer;
                               Var cbBytesNeeded,ServicesReturned: Integer): Bool; StdCall;
  Function EnumDependentServicesW (hService: TSC_HANDLE; ServiceState: Integer;
                               Var Services: TENUM_SERVICE_STATUSW;
                                   cbBufSize: Integer;
                               Var cbBytesNeeded,ServicesReturned: Integer): Bool; StdCall;
  Function EnumDependentServices  (hService: TSC_HANDLE; ServiceState: Integer;
                               Var Services: TENUM_SERVICE_STATUS;
                                   cbBufSize: Integer;
                               Var cbBytesNeeded,ServicesReturned: Integer): Bool; StdCall;
  Function EnumServicesStatusA (hSCManager: TSC_HANDLE; ServiceType, ServiceState: Integer;
                            Var Services: TENUM_SERVICE_STATUSA;
                                cbBufSize: Integer;
                            Var cbBytesNeeded, ServicesReturned, ResumeHandle: Integer): Bool; StdCall;
  Function EnumServicesStatusW (hSCManager: TSC_HANDLE; ServiceType, ServiceState: Integer;
                            Var Services: TENUM_SERVICE_STATUSW;
                                cbBufSize: Integer;
                            Var cbBytesNeeded, ServicesReturned, ResumeHandle: Integer): Bool; StdCall;
  Function EnumServicesStatus  (hSCManager: TSC_HANDLE; ServiceType, ServiceState: Integer;
                            Var Services: TENUM_SERVICE_STATUS;
                                cbBufSize: Integer;
                            Var cbBytesNeeded, ServicesReturned, ResumeHandle: Integer): Bool; StdCall;
  Function GetServiceKeyNameA (hSCManager: TSC_HANDLE; DisplayName, ServiceName: PAnsiChar;
                           Var lpcchBuffer: Integer): Bool; StdCall;
  Function GetServiceKeyNameW (hSCManager: TSC_HANDLE; DisplayName, ServiceName: PWideChar;
                           Var lpcchBuffer: Integer): Bool; StdCall;
  Function GetServiceKeyName  (hSCManager: TSC_HANDLE; DisplayName, ServiceName: PChar;
                           Var lpcchBuffer: Integer): Bool; StdCall;
  Function GetServiceDisplayNameA (hSCManager: TSC_HANDLE; ServiceName, DisplayName: PAnsiChar;
                               Var cchBuffer: Integer): Bool; StdCall;
  Function GetServiceDisplayNameW (hSCManager: TSC_HANDLE; ServiceName, DisplayName: PWideChar;
                               Var cchBuffer: Integer): Bool; StdCall;
  Function GetServiceDisplayName  (hSCManager: TSC_HANDLE; ServiceName, DisplayName: PChar;
                               Var cchBuffer: Integer): Bool; StdCall;
  Function LockServiceDatabase (hSCManager: TSC_HANDLE): TSC_Handle; StdCall;
  Function NotifyBootConfigStatus (BootAcceptable: Bool): Bool; StdCall;

  Function OpenSCManagerA (MachineName, DatabaseName: PAnsiChar; DesiredAccess: Integer): TSC_Handle; StdCall;
  Function OpenSCManagerW (MachineName, DatabaseName: PWideChar; DesiredAccess: Integer): TSC_Handle; StdCall;
  Function OpenSCManager  (MachineName, DatabaseName: PChar; DesiredAccess: Integer): TSC_Handle; StdCall;
  Function OpenServiceA (hSCManager: TSC_HANDLE;lpServiceName: PAnsiChar; DesiredAccess: Integer): TSC_Handle; StdCall;
  Function OpenServiceW (hSCManager: TSC_HANDLE;lpServiceName: PWideChar; DesiredAccess: Integer): TSC_Handle; StdCall;
  Function OpenService  (hSCManager: TSC_HANDLE;lpServiceName: PChar; DesiredAccess: Integer): TSC_Handle; StdCall;
  Function QueryServiceConfigA (hService: TSC_HANDLE; Var ServiceConfig: TQUERY_SERVICE_CONFIGA;
                                cbBufSize: Integer; Var cbBytesNeeded: Integer): Bool; StdCall;
  Function QueryServiceConfigW (hService: TSC_HANDLE; Var ServiceConfig: TQUERY_SERVICE_CONFIGW;
                                cbBufSize: Integer; Var cbBytesNeeded: Integer): Bool; StdCall;
  Function QueryServiceConfig  (hService: TSC_HANDLE; Var ServiceConfig: TQUERY_SERVICE_CONFIG;
                                cbBufSize: Integer; Var cbBytesNeeded: Integer): Bool; StdCall;
  Function QueryServiceLockStatusA (hSCManager: TSC_HANDLE;
                                Var LockStatus: TQUERY_SERVICE_LOCK_STATUSA;
                                    cbBufSize: Integer;
                                Var cbBytesNeeded: Integer): Bool; StdCall;
  Function QueryServiceLockStatusW (hSCManager: TSC_HANDLE;
                                Var LockStatus: TQUERY_SERVICE_LOCK_STATUSW;
                                    cbBufSize: Integer;
                                Var cbBytesNeeded: Integer): Bool; StdCall;
  Function QueryServiceLockStatus  (hSCManager: TSC_HANDLE;
                                Var LockStatus: TQUERY_SERVICE_LOCK_STATUS;
                                    cbBufSize: Integer;
                                Var cbBytesNeeded: Integer): Bool; StdCall;
  Function QueryServiceObjectSecurity (hService: TSC_HANDLE; dwSecurityInformation: TSECURITYINFORMATION;
                                   Var SecurityDescriptor: TSECURITYDESCRIPTOR; cbBufSize: Integer;
                                   Var cbBytesNeeded: Integer): Bool; StdCall;
  Function QueryServiceStatus (hService: TSC_HANDLE; Var ServiceStatus: TSERVICE_STATUS): Bool; StdCall;
  Function RegisterServiceCtrlHandlerA (ServiceName: PAnsiChar; HandlerProc: THANDLER_FUNCTION): TSERVICE_STATUS_HANDLE; StdCall;
  Function RegisterServiceCtrlHandlerW (ServiceName: PWideChar; HandlerProc: THANDLER_FUNCTION): TSERVICE_STATUS_HANDLE; StdCall;
  Function RegisterServiceCtrlHandler (ServiceName: PChar; HandlerProc: THANDLER_FUNCTION): TSERVICE_STATUS_HANDLE; StdCall;
  Function SetServiceObjectSecurity (hService: TSC_HANDLE; SecurityInformation: TSECURITYINFORMATION;
                                 Var SecurityDescriptor: TSECURITYDESCRIPTOR): Bool; StdCall;
  Function SetServiceStatus (hServiceStatus: TSERVICE_STATUS_HANDLE; Var ServiceStatus: TSERVICE_STATUS): Bool; StdCall;
  Function StartServiceCtrlDispatcherA (Var ServiceStartTable{: TSERVICE_TABLE_ENTRYA}): Bool; StdCall;
  Function StartServiceCtrlDispatcherW (Var ServiceStartTable{: TSERVICE_TABLE_ENTRYW}): Bool; StdCall;
  Function StartServiceCtrlDispatcher  (Var ServiceStartTable{: TSERVICE_TABLE_ENTRY}): Bool; StdCall;
  Function StartServiceA (hService: TSC_HANDLE; NumServiceArgs: Integer; Var ServiceArgVectors: PAnsiChar): Bool; StdCall;
  Function StartServiceW (hService: TSC_HANDLE; NumServiceArgs: Integer; Var ServiceArgVectors: PWideChar): Bool; StdCall;
  Function StartService  (hService: TSC_HANDLE; NumServiceArgs: Integer; Var ServiceArgVectors: PChar): Bool; StdCall;
  Function UnlockServiceDatabase (ScLock: TSC_LOCK): Bool; StdCall;

Implementation
Const
  ADVAPI32 = 'ADVAPI32.DLL';

  Function ChangeServiceConfigA;        External ADVAPI32 Name 'ChangeServiceConfigA';
  Function ChangeServiceConfigW;        External ADVAPI32 Name 'ChangeServiceConfigW';
  Function ChangeServiceConfig;         External ADVAPI32 Name 'ChangeServiceConfigA';
  Function CloseServiceHandle;          External ADVAPI32 Name 'CloseServiceHandle';
  Function ControlService;              External ADVAPI32 Name 'ControlService';
  Function CreateServiceA;              External ADVAPI32 Name 'CreateServiceA';
  Function CreateServiceW;              External ADVAPI32 Name 'CreateServiceW';
  Function CreateService;               External ADVAPI32 Name 'CreateServiceA';
  Function DeleteService;               External ADVAPI32 Name 'DeleteService';
  Function EnumDependentServicesA;      External ADVAPI32 Name 'EnumDependentServicesA';
  Function EnumDependentServicesW;      External ADVAPI32 Name 'EnumDependentServicesW';
  Function EnumDependentServices;       External ADVAPI32 Name 'EnumDependentServicesA';
  Function EnumServicesStatusA;         External ADVAPI32 Name 'EnumServicesStatusA';
  Function EnumServicesStatusW;         External ADVAPI32 Name 'EnumServicesStatusW';
  Function EnumServicesStatus;          External ADVAPI32 Name 'EnumServicesStatusA';
  Function GetServiceDisplayNameA;      External ADVAPI32 Name 'GetServiceDisplayNameA';
  Function GetServiceDisplayNameW;      External ADVAPI32 Name 'GetServiceDisplayNameW';
  Function GetServiceDisplayName;       External ADVAPI32 Name 'GetServiceDisplayNameA';
  Function GetServiceKeyNameA;          External ADVAPI32 Name 'GetServiceKeyNameA';
  Function GetServiceKeyNameW;          External ADVAPI32 Name 'GetServiceKeyNameW';
  Function GetServiceKeyName;           External ADVAPI32 Name 'GetServiceKeyNameA';
  Function LockServiceDatabase;         External ADVAPI32 Name 'LockServiceDatabase';
  Function NotifyBootConfigStatus;      External ADVAPI32 Name 'NotifyBootConfigStatus';
  Function OpenSCManagerA;              External ADVAPI32 Name 'OpenSCManagerA';
  Function OpenSCManagerW;              External ADVAPI32 Name 'OpenSCManagerW';
  Function OpenSCManager;               External ADVAPI32 Name 'OpenSCManagerA';
  Function OpenServiceA;                External ADVAPI32 Name 'OpenServiceA';
  Function OpenServiceW;                External ADVAPI32 Name 'OpenServiceW';
  Function OpenService;                 External ADVAPI32 Name 'OpenServiceA';
  Function QueryServiceConfigA;         External ADVAPI32 Name 'QueryServiceConfigA';
  Function QueryServiceConfigW;         External ADVAPI32 Name 'QueryServiceConfigW';
  Function QueryServiceConfig;          External ADVAPI32 Name 'QueryServiceConfigA';
  Function QueryServiceLockStatusA;     External ADVAPI32 Name 'QueryServiceLockStatusA';
  Function QueryServiceLockStatusW;     External ADVAPI32 Name 'QueryServiceLockStatusW';
  Function QueryServiceLockStatus;      External ADVAPI32 Name 'QueryServiceLockStatusA';
  Function QueryServiceObjectSecurity;  External ADVAPI32 Name 'QueryServiceObjectSecurity';
  Function QueryServiceStatus;          External ADVAPI32 Name 'QueryServiceStatus';
  Function RegisterServiceCtrlHandlerA; External ADVAPI32 Name 'RegisterServiceCtrlHandlerA';
  Function RegisterServiceCtrlHandlerW; External ADVAPI32 Name 'RegisterServiceCtrlHandlerW';
  Function RegisterServiceCtrlHandler;  External ADVAPI32 Name 'RegisterServiceCtrlHandlerA';
  Function SetServiceObjectSecurity;    External ADVAPI32 Name 'SetServiceObjectSecurity';
  Function SetServiceStatus;            External ADVAPI32 Name 'SetServiceStatus';
  Function StartServiceCtrlDispatcherA; External ADVAPI32 Name 'StartServiceCtrlDispatcherA';
  Function StartServiceCtrlDispatcherW; External ADVAPI32 Name 'StartServiceCtrlDispatcherW';
  Function StartServiceCtrlDispatcher;  External ADVAPI32 Name 'StartServiceCtrlDispatcherA';
  Function StartServiceA;               External ADVAPI32 Name 'StartServiceA';
  Function StartServiceW;               External ADVAPI32 Name 'StartServiceW';
  Function StartService;                External ADVAPI32 Name 'StartServiceA';
  Function UnlockServiceDatabase;       External ADVAPI32 Name 'UnlockServiceDatabase';
End.

